﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 18.04.2018
 * Time: 05:03
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using HarmonyLib;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol
{
	/// <summary>
	/// Description of ArchitectButtonFix.
	/// </summary> 
	[HarmonyPatch(typeof(MainTabWindow_Architect))]
	[HarmonyPatch("CacheDesPanels")]
	[HarmonyPatchNamespace("ArchitectButtonVisibility")]
	public static class ArchitectButtonFix
	{
		static bool ranOnce = false;
		const string mintsMenuAssembly = "DubsMintMenus, Version=1.2.7123.42234, Culture=neutral, PublicKeyToken=null";
		const string architectIconsAssembly = "ArchitectIcons, Version=1.2.0.0, Culture=en, PublicKeyToken=null";
		
		static bool Prepare(Harmony instance) {
			if ( !ranOnce ) {
				try {
					Type mainTabWindow_ArchitectType = DefSettingsField.GetTypeFromAnyAssemblyVersion("ArchitectIcons.MainTabWindow_Architect, "+architectIconsAssembly);
					if ( mainTabWindow_ArchitectType != null ) {
						instance.Patch(mainTabWindow_ArchitectType.GetMethod("CacheDesPanels", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic),null,
						               new HarmonyMethod(typeof(ArchitectButtonFix).
						                                 GetMethod("PostfixArchitectIcons",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic)),null);
						Log.Message("Compatibility-Patch for Architect Icons is active.");
					}
				} catch {
					Log.Error("Failed to determine if architect icons mod is loaded.");
				}
				try {
					Type mainTabWindow_MintArchitect = DefSettingsField.GetTypeFromAnyAssemblyVersion("DubsMintMenus.MainTabWindow_MintArchitect, "+mintsMenuAssembly);
					if ( mainTabWindow_MintArchitect != null ) {
						instance.Patch(mainTabWindow_MintArchitect.GetMethod("get_CacheDesPanels", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic),null,
						               null,new HarmonyMethod(typeof(ArchitectButtonFix).
						                                 GetMethod("TranspilerMintMenus",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic)));
						Log.Message("Compatibility-Patch for MintMenu architect is active.");
					}
				} catch (Exception ex) {
					Log.Error("Failed to determine if mint menu mod is loaded.\n"+ex.Message+"\n"+ex.StackTrace);
				}
				ranOnce = true;
			}
			return true;
		}
		
		static bool Prefix(MainTabWindow_Architect __instance) {
			try {
                QuickSearchWidget qsw = (QuickSearchWidget)typeof(MainTabWindow_Architect).GetField("quickSearchWidget", BindingFlags.Instance | BindingFlags.NonPublic).GetValue(__instance);

                List<ArchitectCategoryTab> desPanelsCached = new List<ArchitectCategoryTab>();
				foreach (DesignationCategoryDef current in from dc in DefDatabase<DesignationCategoryDef>.AllDefs 
				         where dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>() == null || dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>().visible
					orderby dc.order descending select dc) {
					desPanelsCached.Add(new ArchitectCategoryTab(current, qsw.filter));
				}
				typeof(MainTabWindow_Architect).GetField("desPanelsCached", System.Reflection.BindingFlags.Instance|System.Reflection.BindingFlags.NonPublic).SetValue(__instance,desPanelsCached);
				return false;
			} catch {
				Log.Error("Failed to run architect buttons patch!");
			}
			return true;
		}
		
		static void PostfixArchitectIcons(MainTabWindow_Architect __instance, QuickSearchWidget ___quickSearchWidget) {
			try {
				Type architectCategoryTabCustomOffsetType = DefSettingsField.GetTypeFromAnyAssemblyVersion("ArchitectIcons.ArchitectCategoryTabCustomOffset, "+architectIconsAssembly);
				ConstructorInfo ci = architectCategoryTabCustomOffsetType.GetConstructor(new Type[]{typeof(DesignationCategoryDef),typeof(QuickSearchFilter)});

				List<ArchitectCategoryTab> desPanelsCached = new List<ArchitectCategoryTab>();
				foreach (DesignationCategoryDef current in from dc in DefDatabase<DesignationCategoryDef>.AllDefs 
				         where dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>() == null || dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>().visible
					orderby dc.order descending select dc) {
					desPanelsCached.Add((ArchitectCategoryTab)ci.Invoke(new object[]{current, ___quickSearchWidget.filter}));
				}
				typeof(MainTabWindow_Architect).GetField("desPanelsCached", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic).SetValue(__instance,desPanelsCached);
			} catch (Exception ex) {
				Log.Message(ex.Message);
				Log.Error("Failed to run architect icons compatibility patch!");
			}
		}
		
		static IEnumerable<DesignationCategoryDef> filteredDefs() {
			return from dc in DefDatabase<DesignationCategoryDef>.AllDefs 
				where dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>() == null || dc.GetModExtension<QOLDefModExtension_DesignationCategoryDef>().visible select dc;
		}
		
		static IEnumerable<CodeInstruction> TranspilerMintMenus(IEnumerable<CodeInstruction> instr, ILGenerator gen) {
			MethodInfo miAllDefs = typeof(DefDatabase<DesignationCategoryDef>).GetProperty("AllDefs",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic).GetGetMethod(true);
			foreach ( CodeInstruction ci in instr ) {
				if (ci.opcode == OpCodes.Call && ci.operand is MethodInfo && (MethodInfo)ci.operand == miAllDefs ) {
					yield return new CodeInstruction(OpCodes.Call, typeof(ArchitectButtonFix).GetMethod("filteredDefs", BindingFlags.Static|BindingFlags.NonPublic|BindingFlags.Public));
				} else yield return ci;
			}
		}
	}
}
