﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 16.09.2019
 * Time: 06:31
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using HarmonyLib;
using RimWorld;
using Verse;

namespace RIMMSqol
{
	[HarmonyPatch(typeof(CompPowerTrader))]
	[HarmonyPatch("PowerOn",MethodType.Setter)]
	[HarmonyPatchNamespace("PowerTraderZero")]
	public static class CompPowerTrader_PowerOn
	{
		static void Postfix(CompPowerTrader __instance) {
			if ( Math.Abs(__instance.Props.basePowerConsumption) < float.Epsilon && !__instance.PowerOn && !__instance.parent.IsBrokenDown() && __instance.parent.Spawned && FlickUtility.WantsToBeOn(__instance.parent) ) {
				__instance.PowerOn = true;
			}
		}
	}
	
	[HarmonyPatch(typeof(CompPowerTrader))]
	[HarmonyPatch("ResetPowerVars")]
	[HarmonyPatchNamespace("PowerTraderZero")]
	public static class CompPowerTrader_ResetPowerVars
	{
		static void Postfix(CompPowerTrader __instance) {
			if ( Math.Abs(__instance.Props.basePowerConsumption) < float.Epsilon && !__instance.PowerOn && !__instance.parent.IsBrokenDown() && __instance.parent.Spawned && FlickUtility.WantsToBeOn(__instance.parent) ) {
				__instance.PowerOn = true;
			}
		}
	}
	
	[HarmonyPatch(typeof(CompPowerTrader))]
	[HarmonyPatch("ReceiveCompSignal")]
	[HarmonyPatchNamespace("PowerTraderZero")]
	public static class CompPowerTrader_ReceiveCompSignal
	{
		static void Postfix(CompPowerTrader __instance, string signal) {
			if ( signal == "FlickedOn" ) {
				if ( Math.Abs(__instance.Props.basePowerConsumption) < float.Epsilon && !__instance.PowerOn && !__instance.parent.IsBrokenDown() && __instance.parent.Spawned && FlickUtility.WantsToBeOn(__instance.parent) ) {
					__instance.PowerOn = true;
				}
			}
		}
	}
	
	[HarmonyPatch(typeof(CompPowerTrader))]
	[HarmonyPatch("PostSpawnSetup")]
	[HarmonyPatchNamespace("PowerTraderZero")]
	public static class CompPowerTrader_PostSpawnSetup
	{
		static void Postfix(CompPowerTrader __instance) {
			if ( Math.Abs(__instance.Props.basePowerConsumption) < float.Epsilon && !__instance.PowerOn && !__instance.parent.IsBrokenDown() && __instance.parent.Spawned ) {
				__instance.PowerOn = true;
			}
		}
	}
	
	[HarmonyPatch(typeof(CompBreakdownable))]
	[HarmonyPatch("Notify_Repaired")]
	[HarmonyPatchNamespace("PowerTraderZero")]
	public static class CompBreakdownable_Notify_Repaired
	{
		static void Postfix(CompBreakdownable __instance) {
			CompPowerTrader compPowerTrader = __instance.parent.TryGetComp<CompPowerTrader>();
			if ( compPowerTrader != null ) {
				if ( Math.Abs(compPowerTrader.Props.basePowerConsumption) < float.Epsilon && !compPowerTrader.PowerOn && !compPowerTrader.parent.IsBrokenDown() && compPowerTrader.parent.Spawned && FlickUtility.WantsToBeOn(compPowerTrader.parent) ) {
					compPowerTrader.PowerOn = true;
				}
			}
		}
	}
}
