﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 10.06.2019
 * Time: 17:57
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Reflection;
using HarmonyLib;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
	[HarmonyPatch(typeof(Designator_Dropdown))]
	[HarmonyPatch("ProcessInput")]
	[HarmonyPatchNamespace("DesignatorDropdown")]
	public static class Designator_Dropdown_ProcessInput
	{
		static Func<Designator_Dropdown, Designator> getActiveDesignator = UtilReflection.CreateGetter<Designator_Dropdown, Designator>(typeof(Designator_Dropdown).GetField("activeDesignator", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public));
		static Func<Designator_Build, BuildableDef> getEntDef = UtilReflection.CreateGetter<Designator_Build, BuildableDef>(typeof(Designator_Build).GetField("entDef", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public));
		
		static bool toggle = false;
		static int inputReset = 0, inputDelay = 10000;
		static Designator_Dropdown lastDesignator = null;
		
		static bool Prefix(Designator_Dropdown __instance, Event ev) {
			int positiveTickCount = Environment.TickCount & Int32.MaxValue;
			if ( positiveTickCount < inputReset && lastDesignator == __instance ) {
				toggle = !toggle;
			} else {
				toggle = false;
			}
			inputReset = positiveTickCount + inputDelay;
			lastDesignator = __instance;
			if ( toggle ) {
				Designator desi = getActiveDesignator(__instance);
				if ( desi != null ) {
					Designator_Build desiBuild = desi as Designator_Build;
					if ( desiBuild != null ) {
						BuildableDef thingDef = getEntDef(desiBuild);
						if ( thingDef != null && thingDef.MadeFromStuff ) {
							desi.ProcessInput(ev);
							return false;
						}
					}
				}
			}
			
			return true;
		}
	}

    [HarmonyPatch(typeof(ArchitectCategoryTab))]
    [HarmonyPatch("DesignationTabOnGUI")]
    [HarmonyPatchNamespace("DesignatorDropdown")]
    public static class ArchitectCategoryTab_DesignationTabOnGUI { 

        static FieldInfo fiActiveDesignator = typeof(Designator_Dropdown).GetField("activeDesignator", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);

        static void Prefix(DesignationCategoryDef ___def)
        {
            foreach (Designator des in ___def.ResolvedAllowedDesignators)
            {
                Designator_Dropdown dd = des as Designator_Dropdown;
                if (dd != null)
                {
                    //check if active designator is visible and if not try to replace it
                    Designator activeDesignator = (Designator)fiActiveDesignator.GetValue(dd);
                    if (activeDesignator == null || !activeDesignator.Visible)
                    {
                        foreach (Designator d in dd.Elements)
                        {
                            //Using the first visible designator otherwise the default can be an entry that has not been unlocked.
                            if (d != null && d.Visible)
                            {
                                dd.SetActiveDesignator(d);
                                break;
                            }
                        }
                    }
                }
            }
        }
    }
}
