﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Xml;
using RIMMSqol.extensions;
using RIMMSqol.genericSettings;
using RIMMSqol.renderers;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
    public class Dialog_ImportExportSettings : Dialog_Flow
    {
        protected class SettingsFieldEntry
        {
            public string label;
            public ISettingsFieldProperties field;
            protected bool active;
            public SettingsInstanceEntry owner;
            public bool Active { get => active; set { active = value; if (active) owner.Active = false; } }
        }

        protected class SettingsInstanceEntry
        {
            public string label;
            public ISettingsInstance instance;
            public List<SettingsFieldEntry> fields;
            protected bool active;
            public bool Active { get => active; set { active = value; if (active) foreach (SettingsFieldEntry entry in fields) entry.Active = false; } }
        }

        protected class SettingsPropertiesEntry
        {
            public string label;
            public SettingsProperties props;
            public List<SettingsInstanceEntry> instances;
        }

        protected class SettingsEntry
        {
            public SettingsPropertiesEntry props;
            public SettingsInstanceEntry instance;
            public SettingsFieldEntry field;

            public SettingsEntry(SettingsPropertiesEntry props)
            {
                this.props = props;
                this.label = ColoredText.Colorize(this.props.label, UnityEngine.Color.yellow);
            }

            public SettingsEntry(SettingsInstanceEntry instance)
            {
                this.instance = instance;
                this.label = ColoredText.Colorize(this.instance.label, UnityEngine.Color.green);
            }

            public SettingsEntry(SettingsFieldEntry field)
            {
                this.field = field;
                this.label = this.field.label;
            }

            protected string label;
            public string Label { get => label; }
            public bool HasActive { get => field != null || instance != null; }
            public bool? Active { get => field != null ? field.Active : instance != null ? instance.Active : (bool?)null; set { if (field != null) field.Active = value.Value; else if (instance != null) instance.Active = value.Value; } }
            public float Indent { get => field != null ? 48 : instance != null ? 24 : 0; }
        }

        protected string trnsHintInitial = "RIMMSQoL.ImportExport.HintInitial";
        protected string trnsHintResetCategories = "RIMMSQoL.ImportExport.HintResetCategories";
        protected string trnsHintRequiresAllProperties = "RIMMSQoL.ImportExport.HintRequiresAllProperties";
        protected string trnsNavigationImport = "RIMMSQoL.ImportExport.NavigationImport";
        protected string trnsNavigationExport = "RIMMSQoL.ImportExport.NavigationExport";
        protected string trnsLabelMergeFileSelection = "RIMMSQoL.ImportExport.LabelMergeFileSelection";
        protected string trnsLabelInfo = "RIMMSQoL.ImportExport.LabelInfo";
        protected string trnsLabelStartImport = "RIMMSQoL.ImportExport.LabelStartImport";
        protected string trnsHintImportFailed = "RIMMSQoL.ImportExport.HintImportFailed";
        protected string trnsHintImportSucceded = "RIMMSQoL.ImportExport.HintImportSucceded";

        protected string trnsLabelFilename = "RIMMSQoL.ImportExport.LabelFilename";
        protected string trnsLabelLabel = "RIMMSQoL.ImportExport.LabelLabel";
        protected string trnsLabelDescription = "RIMMSQoL.ImportExport.LabelDescription";
        protected string trnsLabelExportTargets = "RIMMSQoL.ImportExport.LabelExportTargets";
        protected string trnsLabelResetCategories = "RIMMSQoL.ImportExport.LabelResetCategories";
        protected string trnsLabelStartExport = "RIMMSQoL.ImportExport.LabelStartExport";
        protected string trnsLabelQuickSelectExportTargets = "RIMMSQoL.ImportExport.LabelQuickSelectExportTargets";
        protected string trnsAllInstancesIn = "RIMMSQoL.ImportExport.AllInstancesIn";
        protected string trnsAllPropertiesIn = "RIMMSQoL.ImportExport.AllPropertiesIn";
        protected string trnsNoInstancesIn = "RIMMSQoL.ImportExport.NoInstancesIn";
        protected string trnsNoPropertiesIn = "RIMMSQoL.ImportExport.NoPropertiesIn";
        protected string trnsAllCategories = "RIMMSQoL.ImportExport.AllCategories";

        public override Vector2 InitialSize
        {
            get
            {
                return new Vector2((float)Math.Min(UI.screenWidth, 1024), (float)Math.Min(UI.screenHeight, 768));
            }
        }

        public Dialog_ImportExportSettings() : base(null, 500f, 500f)
        {
            this.onlyOneOfTypeAllowed = true;
        }

        public override void PreOpen()
        {
            base.PreOpen();

            trnsHintInitial.TryTranslate(out trnsHintInitial);
            trnsHintResetCategories.TryTranslate(out trnsHintResetCategories);
            trnsHintRequiresAllProperties.TryTranslate(out trnsHintRequiresAllProperties);
            trnsNavigationImport.TryTranslate(out trnsNavigationImport);
            trnsNavigationExport.TryTranslate(out trnsNavigationExport);
            trnsLabelMergeFileSelection.TryTranslate(out trnsLabelMergeFileSelection);
            trnsLabelInfo.TryTranslate(out trnsLabelInfo);
            trnsLabelStartImport.TryTranslate(out trnsLabelStartImport);
            trnsHintImportFailed.TryTranslate(out trnsHintImportFailed);
            trnsHintImportSucceded.TryTranslate(out trnsHintImportSucceded);
            trnsLabelFilename.TryTranslate(out trnsLabelFilename);
            trnsLabelLabel.TryTranslate(out trnsLabelLabel);
            trnsLabelDescription.TryTranslate(out trnsLabelDescription);
            trnsLabelExportTargets.TryTranslate(out trnsLabelExportTargets);
            trnsLabelResetCategories.TryTranslate(out trnsLabelResetCategories);
            trnsLabelStartExport.TryTranslate(out trnsLabelStartExport);
            trnsLabelQuickSelectExportTargets.TryTranslate(out trnsLabelQuickSelectExportTargets);

            trnsAllInstancesIn.TryTranslate(out trnsAllInstancesIn);
            trnsAllPropertiesIn.TryTranslate(out trnsAllPropertiesIn);
            trnsNoInstancesIn.TryTranslate(out trnsNoInstancesIn);
            trnsNoPropertiesIn.TryTranslate(out trnsNoPropertiesIn);
            trnsAllCategories.TryTranslate(out trnsAllCategories);
        }

        protected void updateResetCategorieLabels(Flow f)
        {
            StringBuilder sb = new StringBuilder();
            List<MergeFile> lstMergeFilesSelected = ((List<MergeFile>)f.pageScope["lstMergeFilesSelected"]);
            foreach(MergeFile mf in lstMergeFilesSelected)
            {
                foreach (string propsId in mf.resetCategories)
                {
                    SettingsProperties props = SettingsStorage.getSettingsProperties(propsId);
                    if (props != null)
                    {
                        if ( sb.Length == 0 )
                        {
                            sb.AppendLine(trnsHintResetCategories);
                        }
                        sb.AppendLine(((ISettingsVisualizationMenu)props.visualization).mainMenuButtonLabel);
                    }
                }
            }
            if (lstMergeFilesSelected.Any(mf => mf.requiresAllProperties)) sb.AppendLineIfNotEmpty().AppendLine(trnsHintRequiresAllProperties);
            f.pageScope["infoText"] = sb.ToString();
        }

        public override void DoWindowContents(Rect inRect)
        {
            if ( flow == null )
            {
                float fullWidth = inRect.width;
                Dictionary<string, PageRenderer> pages = new Dictionary<string, PageRenderer>();

                pages.Add("import", new PageRenderer(null, null, null, null, (f, prOld) => {
                    f.pageScope["requiresAllProperties"] = false;
                    f.pageScope["lstResetCategorieLabels"] = new List<string>();
                    f.pageScope["lstMergeFilesAvailable"] = LoadMergeFiles();
                    f.pageScope["lstMergeFilesSelected"] = new List<MergeFile>();
                    f.pageScope["infoText"] = trnsHintInitial;
                })
                {
                    components = new List<BaseRenderer>() {
                    new RowLayoutRenderer().AddChild(
                        new ColumnLayoutRenderer().AddChild(
                            new LabelRenderer(trnsNavigationImport, fullWidth / 2f, GameFont.Medium, TextAnchor.MiddleCenter)).AddChild(
                            new ButtonTextRenderer(trnsNavigationExport, f => { f.navigate("export"); }, GameFont.Medium, fullWidth / 2f))).AddChild(
                        new LabelRenderer(trnsLabelMergeFileSelection, fullWidth, GameFont.Small, UnityEngine.TextAnchor.MiddleCenter)).AddChild(
                        new ListRenderer(f=>(List<MergeFile>)f.pageScope["lstMergeFilesSelected"]).AddChild(
                            new LabelRenderer(f => ((MergeFile)((IterationItem)f.pageScope["curItem"]).curItem).label, fullWidth * 9f / 10f)
                            .attachTooltip(f=>((MergeFile)((IterationItem)f.pageScope["curItem"]).curItem).description)).AddChild(
                            new ButtonTextRenderer("-", f=> {
                                MergeFile lstItem = (MergeFile)((IterationItem)f.pageScope["curItem"]).curItem;
                                f.addPostRenderCallback(fl=>{
                                    ((List<MergeFile>)fl.pageScope["lstMergeFilesSelected"]).Remove(lstItem);
                                    updateResetCategorieLabels(fl);
                                });
                            }, GameFont.Small, fullWidth / 10f)
                            )).AddChild(
                        new ButtonTextRenderer("+", f=>
                            Find.WindowStack.Add(new Dialog_Select<MergeFile>(mf=> {
                                ((List<MergeFile>)f.pageScope["lstMergeFilesSelected"]).Add(mf);
                                updateResetCategorieLabels(f);
                                return true;
                            }, ((List<MergeFile>)f.pageScope["lstMergeFilesAvailable"]).Where(mf => !((List<MergeFile>)f.pageScope["lstMergeFilesSelected"]).Contains(mf)), mf => mf.label, null, mf => mf.description)), GameFont.Small, fullWidth)).AddChild(
                        new LabelRenderer(trnsLabelInfo, fullWidth, GameFont.Small, UnityEngine.TextAnchor.MiddleLeft)).AddChild(
                        new TextRenderer(f=>((string)f.pageScope["infoText"]), null, fullWidth, GameFont.Small, QOLMod.LineHeight(GameFont.Small) * 5)).AddChild(
                        new ButtonTextRenderer(trnsLabelStartImport, f => {
                            if ( !LongEventHandler.AnyEventNowOrWaiting )
                            {
                                LongEventHandler.QueueLongEvent(()=>{
                                    string file = SettingsStorage.BuildConfigFile(QOLMod.getModHandle(),
                                        ((List<MergeFile>)f.pageScope["lstMergeFilesSelected"]));
                                    if ( file != null )
                                    {
                                        f.pageScope["infoText"] = string.Format(trnsHintImportSucceded, file);
                                    }
                                    else
                                    {
                                        f.pageScope["infoText"] = trnsHintImportFailed;
                                    }
                                }, "RIMMSQoL.ImportExport.LabelTaskMerging", true, null, false);
                            }
                        }, GameFont.Medium, fullWidth))
                }
                });
                pages.Add("export", new PageRenderer(null, null, null, null, (f, prOld) => {
                    f.pageScope["filename"] = "my file";
                    f.pageScope["label"] = "my label";
                    f.pageScope["description"] = "my description";
                    f.pageScope["resetCategories"] = new List<SettingsProperties>();
                    //each category with id and label has a list of objects with key and label which in turn have properties with key and label.
                    //each object and property can be toggled on or off.
                    //each checkbox sends a signal with a set of ids which in turn can be used to make the changes in the list via a callback action.
                    List<SettingsPropertiesEntry> lstAvailableProperties = new List<SettingsPropertiesEntry>();
                    foreach (SettingsProperties prop in SettingsStorage.getAllSettingsProperties())
                    {
                        if (!prop.visualization.displayInMenu) continue;

                        List<SettingsInstanceEntry> instances = new List<SettingsInstanceEntry>();
                        foreach (ISettingsInstance si in SettingsStorage.GetInstances(prop))
                        {
                            if (!si.getActive()) continue;
                            List<SettingsFieldEntry> fields = new List<SettingsFieldEntry>();
                            SettingsInstanceEntry sie = new SettingsInstanceEntry() { instance = si, label = si.getLabel(), fields = fields };
                            foreach (ISettingsFieldProperties field in prop.fields)
                            {
                                if (!si.hasFieldConfiguredStatus() || si.isFieldConfigured(field.id))
                                    fields.Add(new SettingsFieldEntry() { field = field, label = field.label, owner = sie });
                            }
                            instances.Add(sie);
                        }
                        lstAvailableProperties.Add(new SettingsPropertiesEntry()
                        {
                            props = prop,
                            label = ((ISettingsVisualizationMenu)prop.visualization).mainMenuButtonLabel,
                            instances = instances
                        });
                    }
                    f.pageScope["availableProperties"] = lstAvailableProperties;
                    List<SettingsEntry> lstAvailablePropertiesStraigthened = new List<SettingsEntry>();
                    foreach (SettingsPropertiesEntry prop in lstAvailableProperties)
                    {
                        lstAvailablePropertiesStraigthened.Add(new SettingsEntry(prop));
                        foreach (SettingsInstanceEntry si in prop.instances)
                        {
                            lstAvailablePropertiesStraigthened.Add(new SettingsEntry(si));
                            foreach (SettingsFieldEntry field in si.fields)
                            {
                                lstAvailablePropertiesStraigthened.Add(new SettingsEntry(field));
                            }
                        }
                    }
                    f.pageScope["availablePropertiesStraigthened"] = lstAvailablePropertiesStraigthened;
                })
                {
                    components = new List<BaseRenderer>()
                {
                    new RowLayoutRenderer().AddChild(
                        new ColumnLayoutRenderer().AddChild(
                            new ButtonTextRenderer(trnsNavigationImport, f => { f.navigate("import"); }, GameFont.Medium, fullWidth / 2f - 2f)).AddChild(
                            new LabelRenderer(trnsNavigationExport, fullWidth / 2f - 2f, GameFont.Medium, TextAnchor.MiddleCenter))).AddChild(
                        new ColumnLayoutRenderer().AddChild(
                            new LabelRenderer(trnsLabelFilename, fullWidth / 4f - 2f, GameFont.Small, UnityEngine.TextAnchor.MiddleLeft)).AddChild(
                            new EditTextRenderer(f=>((string)f.pageScope["filename"]), (f,s)=>f.pageScope["filename"] = s, fullWidth * 3f / 4f - 2f))).AddChild(
                        new ColumnLayoutRenderer().AddChild(
                            new LabelRenderer(trnsLabelLabel, fullWidth / 4f - 2f, GameFont.Small, UnityEngine.TextAnchor.MiddleLeft)).AddChild(
                            new EditTextRenderer(f=>((string)f.pageScope["label"]), (f,s)=>f.pageScope["label"] = s, fullWidth * 3f / 4f - 2f))).AddChild(
                        new LabelRenderer(trnsLabelDescription, fullWidth, GameFont.Small, UnityEngine.TextAnchor.MiddleLeft)).AddChild(
                        new TextRenderer(f=>((string)f.pageScope["description"]), (f,s)=> f.pageScope["description"] = s, fullWidth, GameFont.Small, QOLMod.LineHeight(GameFont.Small) * 5)).AddChild(
                        new ColumnLayoutRenderer().AddChild(
                            new LabelRenderer(trnsLabelExportTargets, fullWidth / 2f - 2f, GameFont.Small, TextAnchor.MiddleCenter)).AddChild(
                            new ButtonTextRenderer(trnsLabelQuickSelectExportTargets, f => {
                                //a floatmenu quickselect that allows to pick [instance/properties] followed by [all,categories..] selects all corresponding entries in the list
                                List<FloatMenuOption> options1 = new List<FloatMenuOption>();
                                List<FloatMenuOption> options2 = new List<FloatMenuOption>();
                                int option1Selected = -1;
                                int option2Selected = -1;
                                options1.Add(new FloatMenuOption(trnsAllInstancesIn, ()=>option1Selected = 0));
                                options1.Add(new FloatMenuOption(trnsAllPropertiesIn, ()=>option1Selected = 1));
                                options1.Add(new FloatMenuOption(trnsNoInstancesIn, ()=>option1Selected = 2));
                                options1.Add(new FloatMenuOption(trnsNoPropertiesIn, ()=>option1Selected = 3));
                                options2.Add(new FloatMenuOption(trnsAllCategories, ()=>option2Selected = 0));
                                int i = 1;
                                foreach ( SettingsProperties sp in SettingsStorage.getAllSettingsProperties().Where(sp => sp.visualization.displayInMenu) )
                                {
                                    int indx = i;
                                    options2.Add(new FloatMenuOption(((ISettingsVisualizationMenu)sp.visualization).mainMenuButtonLabel, 
                                        ()=>option2Selected = indx));
                                    i++;
                                }
                                FloatMenu floatMenu = new FloatMenu(options1);
                                floatMenu.vanishIfMouseDistant = true;
                                floatMenu.onCloseCallback = () => {
                                    if ( option1Selected >= 0 )
                                    {
                                        FloatMenu fm = new FloatMenu(options2);
                                        fm.vanishIfMouseDistant = true;
                                        fm.onCloseCallback = ()=>{
                                            if ( option2Selected < 0 ) return;

                                            SettingsProperties spSelected = null;
                                            if ( option2Selected > 0 )
                                            {
                                                List<SettingsProperties> props = SettingsStorage.getAllSettingsProperties().Where(sp => sp.visualization.displayInMenu).ToList();
                                                if ( props.Count >= option2Selected)
                                                {
                                                    spSelected = props[option2Selected - 1];
                                                }
                                            }
                                            foreach (SettingsPropertiesEntry prop in (List<SettingsPropertiesEntry>)f.pageScope["availableProperties"])
                                            {
                                                if ( spSelected == null || prop.props == spSelected )
                                                {
                                                    foreach (SettingsInstanceEntry si in prop.instances)
                                                    {
                                                        if ( option1Selected == 0 )
                                                        {
                                                            si.Active = true;
                                                            continue;
                                                        }
                                                        else if ( option1Selected == 2 )
                                                        {
                                                            si.Active = false;
                                                            continue;
                                                        }
                                                        foreach (SettingsFieldEntry field in si.fields)
                                                        {
                                                            if ( option1Selected == 1 ) field.Active = true;
                                                            else if ( option1Selected == 3 ) field.Active = false;
                                                        }
                                                    }
                                                }
                                            }
                                        };
                                        Find.WindowStack.Add(fm);
                                    }
                                };
                                Find.WindowStack.Add(floatMenu);
                            }, GameFont.Small, fullWidth / 2f - 2f))).AddChild(
                        new ListRenderer(f=>(List<SettingsEntry>)f.pageScope["availablePropertiesStraigthened"], fullWidth, 
                            inRect.height - QOLMod.LineHeight(GameFont.Small) * 11f - 100f - QOLMod.LineHeight(GameFont.Medium) * 2f - 4f * 10f - 16f).AddChild(
                            new CheckboxRenderer(f=> {
                                    SettingsEntry se = (SettingsEntry)((IterationItem)f.pageScope["curItem"]).curItem;
                                    return se.HasActive ? se.Active.Value : false;
                                }, (f,b)=>((SettingsEntry)((IterationItem)f.pageScope["curItem"]).curItem).Active = b) { disabledExpr = f => !((SettingsEntry)((IterationItem)f.pageScope["curItem"]).curItem).HasActive }).AddChild(
                            new LabelRenderer(f=>((SettingsEntry)((IterationItem)f.pageScope["curItem"]).curItem).Label, fullWidth - 24)
                            )).AddChild(
                        new LabelRenderer(trnsLabelResetCategories, fullWidth, GameFont.Small, UnityEngine.TextAnchor.MiddleCenter)).AddChild(
                        new ListRenderer(f=>(List<SettingsProperties>)f.pageScope["resetCategories"], fullWidth).AddChild(
                            new LabelRenderer(f => ((ISettingsVisualizationMenu)((SettingsProperties)((IterationItem)f.pageScope["curItem"]).curItem).visualization).mainMenuButtonLabel, fullWidth * 9f / 10f)).AddChild(
                            new ButtonTextRenderer("-", f=> {
                                SettingsProperties lstItem = (SettingsProperties)((IterationItem)f.pageScope["curItem"]).curItem;
                                f.addPostRenderCallback(fl=>{
                                    ((List<SettingsProperties>)f.pageScope["resetCategories"]).Remove(lstItem);
                                });
                            }, GameFont.Small, fullWidth / 10f)
                            )).AddChild(
                        new ButtonTextRenderer("+", f=>
                            Find.WindowStack.Add(new Dialog_Select<SettingsProperties>(sp=> {
                                ((List<SettingsProperties>)f.pageScope["resetCategories"]).Add(sp);
                                return true;
                            }, SettingsStorage.getAllSettingsProperties().Where(sp => sp.visualization.displayInMenu), sp =>((ISettingsVisualizationMenu)sp.visualization).mainMenuButtonLabel)), GameFont.Small, fullWidth)).AddChild(
                        new ButtonTextRenderer(trnsLabelStartExport, f => {
                                if ( !LongEventHandler.AnyEventNowOrWaiting )
                                    LongEventHandler.QueueLongEvent(()=>
                                        BuildMergeFile(
                                            (string)f.pageScope["filename"], 
                                            (string)f.pageScope["label"], 
                                            (string)f.pageScope["description"], 
                                            (List<SettingsPropertiesEntry>)f.pageScope["availableProperties"], 
                                            (List<SettingsProperties>)f.pageScope["resetCategories"], 
                                            false),
                                        "RIMMSQoL.ImportExport.LabelTaskCreatingMergeFile", true, null, false);
                            }, GameFont.Medium, fullWidth))
                }
                });

                this.flow = new Flow(pages, "import");
            }
            base.DoWindowContents(inRect);
        }

        protected List<MergeFile> LoadMergeFiles()
        {
            List<MergeFile> mergeFiles = new List<MergeFile>();

            DirectoryInfo directoryInfo = new DirectoryInfo(MergeFile.GetMergeFileDirectory());
            if (directoryInfo.Exists)
            {
                foreach (FileInfo fileInfo in directoryInfo.GetFiles("*.xml", SearchOption.TopDirectoryOnly))
                {
                    Scribe.loader.InitLoading(fileInfo.FullName);
                    try
                    {
                        MergeFile mf = new MergeFile();
                        Scribe_Deep.Look<MergeFile>(ref mf, "MergeFile", Array.Empty<object>());
                        if (mf != null) mergeFiles.Add(mf);
                        else Log.Error("Failed to load merge file: " + fileInfo.FullName);
                    }
                    finally
                    {
                        Scribe.loader.FinalizeLoading();
                    }
                }
            }

            return mergeFiles;
        }

        protected void BuildMergeFile(string filename, string label, string description, List<SettingsPropertiesEntry> availableProperties, List<SettingsProperties> resetCategories, bool requiresAllProperties)
        {
            Mod mod = QOLMod.getModHandle();
            string filePath = Path.Combine(MergeFile.GetMergeFileDirectory(mod), GenText.SanitizeFilename(string.Format(filename + ".xml")));

            MergeFile mergeFile = new MergeFile() { label = label, description = description, requiresAllProperties = requiresAllProperties };
            List<string> resetCategoriesAsString = new List<string>();
            foreach (SettingsProperties props in resetCategories)
            {
                resetCategoriesAsString.Add(props.id);
            }
            mergeFile.resetCategories = resetCategoriesAsString;
            List<PatchOperation> patches = new List<PatchOperation>();

            XmlDocument docExistingSettings = new XmlDocument();
            docExistingSettings.Load(Path.Combine(GenFilePaths.ConfigFolderPath, GenText.SanitizeFilename(string.Format("Mod_{0}_{1}.xml", mod.Content.FolderName, mod.GetType().Name))));

            foreach (SettingsPropertiesEntry spe in availableProperties)
            {
                foreach (SettingsInstanceEntry sie in spe.instances)
                {
                    if (sie.Active || sie.fields.Any(e=>e.Active))
                    {
                        string xpath = "/SettingsBlock/ModSettings/" + spe.props.id + "/li[./baseObjectKey[normalize-space() = '" + sie.instance.getKey() + "']]";
                        PatchOperation patch = null;
                        XmlContainer xmlContainer = new XmlContainer();
                        if (!sie.Active)
                        {
                            //we add active properties to the instance
                            patch = new PatchOperation_QoLSetProperty() { xpath = xpath, value = xmlContainer };
                        }
                        else
                        {
                            //we overwrite the instance
                            patch = new PatchOperation_QoLSetInstance() { xpath = xpath, value = xmlContainer };
                        }

                        //we ignore escaping of def names since I doubt the rest of the scribe world would be able to cope anyway
                        XmlNode existingNode = docExistingSettings.SelectSingleNode(xpath);
                        if ( existingNode != null )
                        {
                            XmlDocument docPatch = new XmlDocument();
                            docPatch.LoadXml("<value></value>");
                            xmlContainer.node = docPatch.DocumentElement;
                            XmlNode patchNode = xmlContainer.node;

                            foreach (SettingsFieldEntry sfe in sie.fields.Where(e => e.Active || sie.Active))
                            {
                                XmlNode n = existingNode[sfe.field.id];
                                if (n != null)
                                {
                                    patchNode.AppendChild(docPatch.ImportNode(n, true));
                                    n = existingNode[sfe.field.id + "_type"];
                                    if (n != null)
                                    {
                                        patchNode.AppendChild(docPatch.ImportNode(n, true));
                                    }
                                }
                                else
                                {
                                    //if the defaultValue equals the value then even a configured property might not appear(primitive values and defs)
                                    //we need to make sure that when imported the default value is overwriting possibly existing settings.
                                    n = existingNode["isConfigured"];
                                    if (n == null || Regex.IsMatch(n.InnerText, "(^|;)" + Regex.Escape(sfe.field.id) + ";t;"))
                                    {
                                        //value is null in a configured proeprty
                                        XmlElement ele = docPatch.CreateElement(sfe.field.id);
                                        ele.SetAttribute("IsDefault", null);
                                        patchNode.AppendChild(ele);
                                    }
                                    else Log.Error("Trying to export unconfigured property: " + sfe.field.id);
                                }
                            }
                            patches.Add(patch);
                        }
                        else
                        {
                            Log.Error("Failed to lookup xpath in settings file: "+xpath);
                        }
                    }
                }
            }
            mergeFile.patches = patches;

            Scribe.saver.InitSaving(filePath, "SettingsBlock");
            try
            {
                Scribe_Deep.Look<MergeFile>(ref mergeFile, "MergeFile", Array.Empty<object>());
            }
            finally
            {
                Scribe.saver.FinalizeSaving();
            }
        }

        protected void AppendTextNode(XmlDocument doc, XmlNode parent, string name, string text)
        {
            XmlElement ele = doc.CreateElement(string.Empty, name, string.Empty);
            ele.AppendChild(doc.CreateTextNode(text));
            parent.AppendChild(ele);
        }
    }
}
