﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 09.02.2019
 * Time: 02:38
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
	/// <summary>
	/// Description of Dialog_SetNumericValue.
	/// </summary>
	public class Dialog_Spinner : Window {
		protected int value, step;
		protected Func<int,bool> validator;
		protected Action<int> setter;
		protected string info, title;
		protected float infoHeight;
		protected GameFont infoFont;
		
		public override Vector2 InitialSize {
			get {
				Vector2 titleSize = QOLMod.CalcSize(title, GameFont.Medium, TextAnchor.MiddleCenter, false);
				return new Vector2(Mathf.Max(140f,titleSize.x)+Margin*2f,QOLMod.LineHeight(GameFont.Medium)+Margin*2f+(title!=null ? titleSize.y+8f : 0f)+infoHeight);
			}
		}
		
		public Dialog_Spinner(int value, Func<int,bool> validator, Action<int> setter, int step = 1, string title = null, string info = null) {
			this.closeOnCancel = true;
			this.doCloseX = true;
			this.onlyOneOfTypeAllowed = true;
			this.absorbInputAroundWindow = true;
			this.forcePause = true;
			this.title = title;
			this.value = value;
			this.step = step;
			this.validator = validator;
			this.setter = setter;
			this.info = info;
			//The dialog window should be at least as wide as the title or 140f pixels + margin*2.
			//It should be no taller than it's wide. Shrinking the font for the info part to small, then tiny and only then the height gets extended past the width.
			float innerWidth = Mathf.Max(140f,QOLMod.CalcSize(this.title, GameFont.Medium, TextAnchor.MiddleCenter, false).x);
			this.infoHeight = QOLMod.CalcHeight(this.info, Mathf.Max(140f,innerWidth), GameFont.Medium);
			if ( infoHeight > innerWidth ) {
				this.infoHeight = QOLMod.CalcHeight(this.info, Mathf.Max(140f,innerWidth), GameFont.Small);
				if ( infoHeight > innerWidth ) {
					this.infoHeight = QOLMod.CalcHeight(this.info, Mathf.Max(140f,innerWidth), GameFont.Tiny);
					infoFont = GameFont.Tiny;
				} else {
					infoFont = GameFont.Small;
				}
			} else {
				infoFont = GameFont.Medium;
			}
		}
		
		override public void PreOpen() {
			base.PreOpen();
		}
		
		public override void PostClose() {
			base.PostClose();
			UI.UnfocusCurrentControl();
			setter(value);
		}
		
		public override void DoWindowContents(Rect inRect) {
			TextAnchor oldAnchor = Text.Anchor;
			GameFont oldFont = Text.Font;
			Text.Anchor = TextAnchor.MiddleCenter;
			Text.Font = GameFont.Medium;
			
			float titleBarHeight = Text.LineHeight+8f;
			Widgets.Label(new Rect(inRect.x, inRect.y + 4f, inRect.width, titleBarHeight-4f),title);
			
			Rect rect = new Rect(inRect.x,inRect.y + titleBarHeight,32f,Text.LineHeight);
			
			if ( Widgets.ButtonText(rect, "-") ) {
				if ( validator(value-step) ) {
					value -= step;
				}
			}
			rect.x += rect.width + 8f; rect.width = inRect.width - 80f;
			Widgets.Label(rect, value.ToString("###0"));
			rect.x += rect.width + 8f; rect.width = 32f;
			if ( Widgets.ButtonText(rect, "+") ) {
				if ( validator(value+step) ) {
					value += step;
				}
			}
			
			if ( info != null ) {
				Text.Font = infoFont;
				Text.Anchor = TextAnchor.UpperLeft;
				Widgets.Label(new Rect(inRect.x, rect.yMax, inRect.width, infoHeight), info);
			}
			
			Text.Anchor = oldAnchor;
			Text.Font = oldFont;
		}
	}
}
