﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 03.09.2019
 * Time: 16:27
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using HarmonyLib;
using RIMMSqol.genericSettings;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
	[HarmonyPatch(typeof(MainTabWindow_Research))]
	[HarmonyPatch("PostOpen")]
	[HarmonyPatchNamespace("ResearchTabDrawer")]
	static public class MainTabWindow_Research_DrawRightRect
	{
		static Func<MainTabWindow_Research, ResearchTabDef> GetCurrentTab = UtilReflection.CreateGetter<MainTabWindow_Research, ResearchTabDef>(
			typeof(MainTabWindow_Research).GetProperty("CurTab", BindingFlags.NonPublic|BindingFlags.Instance));
		static Action<MainTabWindow_Research, ResearchTabDef> SetCurrentTab = UtilReflection.CreateSetter<MainTabWindow_Research, ResearchTabDef>(
			typeof(MainTabWindow_Research).GetProperty("CurTab", BindingFlags.NonPublic|BindingFlags.Instance));
		
		static System.Collections.IList lstTabRecords = new List<TabRecord>(0);
		static List<ResearchTabDef> lstResearchTabs = new List<ResearchTabDef>(0);
		static bool isDirty = true;
		static object monitor = new object();
		static MainTabWindow_Research cachedInstance;
		
		public static void ClearCachedData() {
			isDirty = true;
			isDirtyResearchTabDefs = true;
		}
		
		static object getTabs(MainTabWindow_Research instance) {
            //Instance changes between games, which must trigger a refresh since the cached delegates reference the instance. 
            isDirty |= true;// cachedInstance != instance;
			if ( isDirty ) {
				lock(monitor) {
					if ( isDirty ) {
                        Type rtr = typeof(MainTabWindow_Research).GetNestedType("ResearchTabRecord", BindingFlags.NonPublic | BindingFlags.Public);
                        List<object[]> sortedTabs = new List<object[]>();
						bool selectedIsVisible = false;
						ResearchTabDef currentlySelected = GetCurrentTab(instance);
						foreach (ResearchTabDef current in DefDatabase<ResearchTabDef>.AllDefs) {
                            QOLDefModExtension_ResearchTabDef modExtension = current.GetModExtension<QOLDefModExtension_ResearchTabDef>();

                            if (modExtension != null && modExtension.alwaysHidden) continue;
                            if ( !DefDatabase<ResearchProjectDef>.AllDefsListForReading.Any(rp=>rp.tab == current) ) {
								continue;
							}

							if ( currentlySelected == current ) selectedIsVisible = true;
							
							int order;
							if ( modExtension != null ) {
								order = modExtension.order;
							} else order = 0;

                            object researchTabRecord = rtr.GetConstructor(new Type[] { typeof(ResearchTabDef), typeof(string), typeof(Action), typeof(Func<bool>) })
                                .Invoke(new object[] { current, current.LabelCap.ToString(), new Action(delegate { SetCurrentTab(instance, current); }), new Func<bool>(delegate { return GetCurrentTab(instance) == current; }) });

							sortedTabs.Add(new object[]{order, researchTabRecord, current});
						}
						sortedTabs.Sort((p1,p2)=> {
		                	int order = ((int)p1[0]).CompareTo((int)p2[0]);
				        	if ( order == 0 )
				        		order = string.Compare(((TabRecord)p1[1]).label, ((TabRecord)p2[1]).label, StringComparison.CurrentCultureIgnoreCase);
				        	return order;
				        });

                        System.Collections.IList lst = (System.Collections.IList)
                            typeof(List<>).MakeGenericType(new Type[] { rtr }).GetConstructor(new Type[] { typeof(int) }).Invoke(new object[] { sortedTabs.Count });
						List<ResearchTabDef> lst2 = new List<ResearchTabDef>(sortedTabs.Count);
						foreach ( object[] p in sortedTabs ) {
							if ( !selectedIsVisible ) {
								SetCurrentTab(instance,(ResearchTabDef)p[2]);
							}
							lst.Add((TabRecord)p[1]);
							lst2.Add((ResearchTabDef)p[2]);
						}
						lstTabRecords = lst;
						lstResearchTabs = lst2;
						isDirty = false;
						cachedInstance = instance;
					}
				}
			}
			
			return lstTabRecords;
		}
		
		static void Postfix(MainTabWindow_Research __instance) {
			typeof(MainTabWindow_Research).GetField("tabs", BindingFlags.NonPublic|BindingFlags.Instance).SetValue(__instance,getTabs(__instance));
		}
		
		static bool ranOnce = false;
		const string mintsMenuAssembly = "DubsMintMenus, Version=1.2.7123.42234, Culture=neutral, PublicKeyToken=null";
		const string researchScrollbarsAssembly = "SupesResearchScrollbars, Version=1.4.0.0, Culture=neutral, PublicKeyToken=null";
		static bool isDirtyResearchTabDefs = false;
		static List<ResearchTabDef> lstResearchTabDefs;
		static object monitorResearchTabDefs = new object();
		static bool Prepare(Harmony instance) {
			if ( !ranOnce ) {
				try {
					Type mainTabWindow_MintResearch = DefSettingsField.GetTypeFromAnyAssemblyVersion("DubsMintMenus.MainTabWindow_MintResearch, "+mintsMenuAssembly);
					if ( mainTabWindow_MintResearch != null ) {
						instance.Patch(mainTabWindow_MintResearch.GetMethod("DrawModsList", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic),null,
						               null,new HarmonyMethod(typeof(MainTabWindow_Research_DrawRightRect).
						                                 GetMethod("TranspilerMintMenus",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic)));
                        instance.Patch(mainTabWindow_MintResearch.GetMethod("parc", BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic), null,
                                       new HarmonyMethod(typeof(MainTabWindow_Research_DrawRightRect).
                                                         GetMethod("PostfixMintMenus", BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic)));
                        Log.Message("Compatibility-Patch for MintMenu research is active.");
					}
				} catch (Exception ex) {
					Log.Error("Failed to determine if mint menu mod is loaded.\n"+ex.Message+"\n"+ex.StackTrace);
				}
				try {
					Type mainTabWindow_ResearchScrollbars = DefSettingsField.GetTypeFromAnyAssemblyVersion("SupesResearchScrollbars.SupesMainTabWindow_ScrollingResearch, "+researchScrollbarsAssembly);
					if ( mainTabWindow_ResearchScrollbars != null ) {
						instance.Patch(mainTabWindow_ResearchScrollbars.GetMethod("DrawRightRect", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic),null,
						               null,new HarmonyMethod(typeof(MainTabWindow_Research_DrawRightRect).
						                                 GetMethod("TranspilerResearchScrollbars",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic)));
						Log.Message("Compatibility-Patch for ResearchScrollbars is active.");
					}
				} catch (Exception ex) {
					Log.Error("Failed to determine if ResearchScrollbars mod is loaded.\n"+ex.Message+"\n"+ex.StackTrace);
				}
				ranOnce = true;
			}
			return true;
		}
		
		static List<ResearchTabDef> getTabDefsNoArgs() {
			return getTabDefs(null);
		}
		
		static List<ResearchTabDef> getTabDefs(MainTabWindow_Research instance) {
			if ( isDirtyResearchTabDefs ) {
				lock(monitorResearchTabDefs) {
					if ( isDirtyResearchTabDefs ) {
						List<object[]> sortedTabs = new List<object[]>();
						foreach (ResearchTabDef current in DefDatabase<ResearchTabDef>.AllDefs) {
                            QOLDefModExtension_ResearchTabDef modExtension = current.GetModExtension<QOLDefModExtension_ResearchTabDef>();

                            if (modExtension != null && modExtension.alwaysHidden) continue;
                            if ( !DefDatabase<ResearchProjectDef>.AllDefsListForReading.Any(rp=>rp.tab == current) ) {
								continue;
							}
							
							int order;
							if ( modExtension != null ) {
								order = modExtension.order;
							} else order = 0;
							
							sortedTabs.Add(new object[]{order,current});
						}
						sortedTabs.Sort((p1,p2)=> {
		                	int order = ((int)p1[0]).CompareTo((int)p2[0]);
				        	if ( order == 0 )
				        		order = string.Compare(((ResearchTabDef)p1[1]).LabelCap, ((ResearchTabDef)p2[1]).LabelCap, StringComparison.CurrentCultureIgnoreCase);
				        	return order;
				        });
						List<ResearchTabDef> lst = new List<ResearchTabDef>(sortedTabs.Count);
						foreach ( object[] p in sortedTabs ) {
							lst.Add((ResearchTabDef)p[1]);
						}
						lstResearchTabDefs = lst;
						isDirtyResearchTabDefs = false;
					}
				}
			}
			
			return lstResearchTabDefs;
		}
		
		static IEnumerable<CodeInstruction> TranspilerMintMenus(IEnumerable<CodeInstruction> instr, ILGenerator gen) {
			MethodInfo miAllDefs = typeof(DefDatabase<ResearchTabDef>).GetProperty("AllDefsListForReading",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic).GetGetMethod(true);
			foreach ( CodeInstruction ci in instr ) {
				if (ci.opcode == OpCodes.Call && ci.operand is MethodInfo && (MethodInfo)ci.operand == miAllDefs ) {
					yield return new CodeInstruction(OpCodes.Ldarg_0);
					yield return new CodeInstruction(OpCodes.Callvirt, typeof(MainTabWindow_Research_DrawRightRect).GetMethod("getTabDefs", BindingFlags.NonPublic|BindingFlags.Static));
				} else yield return ci;
			}
		}
		
		static IEnumerable<CodeInstruction> TranspilerResearchScrollbars(IEnumerable<CodeInstruction> instr, ILGenerator gen) {
			MethodInfo miAllDefs = typeof(DefDatabase<ResearchTabDef>).GetProperty("AllDefs",BindingFlags.Static|BindingFlags.Public|BindingFlags.NonPublic).GetGetMethod(true);
			foreach ( CodeInstruction ci in instr ) {
				if ( ci.opcode == OpCodes.Call && ci.operand is MethodInfo && (MethodInfo)ci.operand == miAllDefs ) {
					yield return new CodeInstruction(OpCodes.Ldarg_0);
					yield return new CodeInstruction(OpCodes.Callvirt, typeof(MainTabWindow_Research_DrawRightRect).GetMethod("getTabDefs", BindingFlags.NonPublic|BindingFlags.Static));
				} else yield return ci;
			}
		}

        static void PostfixMintMenus(ResearchProjectDef p, ref bool __result)
        {
            //reverse visibility of project if it's inside a hidden research tab
            if (__result && p != null && p.tab != null && !getTabDefsNoArgs().Contains(p.tab))
                __result = false;
        }
	}
}
