﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 31.05.2017
 * Time: 15:42
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using Verse;
using UnityEngine;
using System.IO;

namespace RIMMSqol
{
	/// <summary>
	/// Description of QOLModSettings.
	/// </summary>
	public class QOLModSettings : ModSettings
	{
        public bool stopSkillDecay = false, useFixedNumericTextfields = false, preventAnimalFamilies = false, stopTamenessDecay = false, avoidDesignatorRefresh = false,
            reapplyOnGameLoad = false, displayFoundCityGizmo = true, stopRoofCollapse = false, stopSkillDown = false, growLists = false, alwaysDisplayLabelInMBDD = true,
            allowNamedAnimalFamilies = false;
		public float cumBaseCost = 20f, cumTax = 0f, cumPointPawnToPoolConversionFactor = 0f, cumPointRemnantsToPoolConversionFactor = 0.25f, remnantOrderPriceFactor = 0.5f,
			starvingThreshold = 0.4f, tooltipMaxWidth = 260f;
		public int skillDecayLevelOffset = 0;
		public List<string> cumTraitCosts;
		public List<string> cumRecordsFactor;
		public List<string> pfRestrictionExcemptions = null;
		public HashSet<string> hiddenAlerts = null;
		public string pfColonist = null, pfAnimalTame = null, pfAnimalWild = null, pfOther = null;
		public string pfColonistConf = null, pfAnimalTameConf = null, pfAnimalWildConf = null, pfOtherConf = null;
		public List<string> forbiddenPatchNamespaces = null;
        public List<QOLModSettingsDefCopy> defCopies = null;
        public DateStyleDef dateStyle = null;
		
		public override void ExposeData()
		{
			base.ExposeData();

            if ( Scribe.mode == LoadSaveMode.LoadingVars )
            {
                if ( Scribe.EnterNode("settings") )
                {
                    if ( Scribe.EnterNode("li") )
                    {
                        Scribe_Collections.Look<string>(ref forbiddenPatchNamespaces, "ForbiddenPatches");
                        Scribe_Collections.Look<QOLModSettingsDefCopy>(ref defCopies, "DefCopies", LookMode.Deep);
                        Scribe.ExitNode();
                    }
                    Scribe.ExitNode();
                }
            }

            //Skip the bulk of the settings so we can load the parts required before defs are resolved.
            //Later we load the settings again and utilize existing defs in the property setups.
            if ( SettingsInit.IsInitialized )
                SettingsStorage.ExposeData();
        }
	}

    public class QOLModSettingsFull : QOLModSettings
    {
        public QOLModSettingsFull(QOLModSettings settings)
        {
            forbiddenPatchNamespaces = settings.forbiddenPatchNamespaces;
            defCopies = settings.defCopies;
        }

        public override void ExposeData()
        {
            if (Scribe.mode == LoadSaveMode.LoadingVars)
            {
                if (Scribe.EnterNode("settings"))
                {
                    if (Scribe.EnterNode("li"))
                    {
                        Scribe_Collections.Look<string>(ref forbiddenPatchNamespaces, "ForbiddenPatches");
                        Scribe_Collections.Look<QOLModSettingsDefCopy>(ref defCopies, "DefCopies", LookMode.Deep);
                        Scribe.ExitNode();
                    }
                    Scribe.ExitNode();
                }
            }

            SettingsStorage.ExposeDataFull();
        }
    }

    public class QOLModSettingsDefCopy : IExposable
    {
        public string defType, defNameOriginal, defNameCopy, labelCopy;

        public void ExposeData()
        {
            Scribe_Values.Look<string>(ref defType, "DefType");
            Scribe_Values.Look<string>(ref defNameOriginal, "DefNameOriginal");
            Scribe_Values.Look<string>(ref defNameCopy, "DefNameCopy");
            Scribe_Values.Look<string>(ref labelCopy, "LabelCopy");
        }

        public bool IsValid
        {
            get
            {
                return !defType.NullOrEmpty() && !defNameOriginal.NullOrEmpty() && !defNameCopy.NullOrEmpty() && labelCopy != null;
            }
        }

        public override string ToString()
        {
            return "defType: "+ defType+" defNameOriginal: "+ defNameOriginal+ " defNameCopy: "+ defNameCopy+ " labelCopy: " + labelCopy;
        }
    }
}
