﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Verse;

namespace RIMMSqol
{
    public class Scribe_DirectObjectLoader
    {
        static Func<ScribeSaver, XmlWriter> getScribeSaver_Writer = UtilReflection.CreateGetter<ScribeSaver, XmlWriter>(typeof(ScribeSaver).GetField("writer", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public));

        public static void Look<T>(ref T value, string label, T defaultValue = default(T))
        {
            if (Scribe.mode == LoadSaveMode.Saving)
            {
                if (value == null)
                {
                    if (!Scribe.EnterNode(label))
                    {
                        try
                        {
                            Scribe.saver.WriteAttribute("IsNull", "True");
                        }
                        finally
                        {
                            Scribe.ExitNode();
                        }
                    }
                }
                else
                {
                    IExposable exposable = value as IExposable;
                    if (exposable != null)
                    {
                        Scribe_Deep.Look(ref exposable, label);
                    }
                    else
                    {
                        if (Scribe.EnterNode(label))
                        {
                            try
                            {
                                XmlContainer xmlContainer = value as XmlContainer;
                                if (xmlContainer != null)
                                {
                                    xmlContainer.node.WriteContentTo(getScribeSaver_Writer(Scribe.saver));
                                }
                            }
                            catch (OutOfMemoryException)
                            {
                                throw;
                            }
                            catch (Exception ex)
                            {
                                Log.Error(string.Concat(new object[] { "Exception while saving ", value.ToStringSafe(), ": ", ex }));
                            }
                            finally
                            {
                                Scribe.ExitNode();
                            }
                        }
                    }
                }
            }
            else if (Scribe.mode == LoadSaveMode.LoadingVars)
            {
                XmlNode n = Scribe.loader.curXmlParent[label];
                if (n != null) value = DirectXmlToObject.ObjectFromXml<T>(n, false);
                else value = defaultValue;
            }
            else if (Scribe.mode == LoadSaveMode.ResolvingCrossRefs)
            {
                DirectXmlCrossRefLoader.ResolveAllWantedCrossReferences(FailMode.LogErrors);
            }
        }

        public static void LookList<T>(ref List<T> value, string label, List<T> defaultValue = default(List<T>))
        {
            if (Scribe.mode == LoadSaveMode.Saving)
            {
                if (value == null)
                {
                    if (!Scribe.EnterNode(label))
                    {
                        try
                        {
                            Scribe.saver.WriteAttribute("IsNull", "True");
                        }
                        finally
                        {
                            Scribe.ExitNode();
                        }
                    }
                }
                else
                {
                    if (Scribe.EnterNode(label))
                    {
                        try
                        {
                            foreach(T item in value)
                            {
                                IExposable exposable = item as IExposable;
                                if (exposable != null)
                                {
                                    Scribe_Deep.Look(ref exposable, "li");
                                }
                                else
                                {
                                    XmlContainer xmlContainer = value as XmlContainer;
                                    if (xmlContainer != null)
                                    {
                                        xmlContainer.node.WriteContentTo(getScribeSaver_Writer(Scribe.saver));
                                    }
                                }
                            }
                        }
                        catch (OutOfMemoryException)
                        {
                            throw;
                        }
                        catch (Exception ex)
                        {
                            Log.Error(string.Concat(new object[] { "Exception while saving ", value.ToStringSafe(), ": ", ex }));
                        }
                        finally
                        {
                            Scribe.ExitNode();
                        }
                    }
                }
            }
            if (Scribe.mode == LoadSaveMode.LoadingVars)
            {
                XmlNode nList = Scribe.loader.curXmlParent[label];
                if (nList != null)
                {
                    if (value == null) value = new List<T>();
                    else value.Clear();
                    foreach (XmlNode n in nList)
                    {
                        T entry = DirectXmlToObject.ObjectFromXml<T>(n, false);
                        value.Add(entry);
                    }
                }
                else value = defaultValue;
            }
            else if (Scribe.mode == LoadSaveMode.ResolvingCrossRefs)
            {
                DirectXmlCrossRefLoader.ResolveAllWantedCrossReferences(FailMode.LogErrors);
            }
        }
    }
}
