﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 01.09.2019
 * Time: 16:16
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Text;
using HarmonyLib;
using Verse;

namespace RIMMSqol
{
	/// <summary>
	/// Description of UtilHarmonyReporting.
	/// </summary>
	static public class UtilHarmonyReporting
	{
		public static string ReportPatchesOnMethod(Type declaringType, string methodName) {
			try {
				StringBuilder sb = new StringBuilder();
				IEnumerable<MethodBase> patchedMethods = PatchProcessor.GetAllPatchedMethods();
				foreach ( MethodBase mb in patchedMethods ) {
					if ( mb.DeclaringType == declaringType && mb.Name == methodName ) {
						sb.Append(declaringType.Namespace).Append(".").Append(declaringType.Name).Append(".").Append(methodName).Append(" ");
						Patches pi = PatchProcessor.GetPatchInfo(mb);
						sb.AppendPatches("PRE", pi.Prefixes);
						sb.AppendPatches("POST", pi.Postfixes);
						sb.AppendPatches("TRANS", pi.Transpilers);
					}
				}
				return sb.ToString();
			} catch {
				return "Failed to log Harmony-Patches on: "+declaringType+"."+methodName;
			}
		}
		
		private static void AppendPatches(this StringBuilder sb, string sectionTitle, ReadOnlyCollection<Patch> patches) {
			if ( patches == null || patches.Count == 0 ) return;
			List<Patch> sortedPatches = new List<Patch>(patches);
			sortedPatches.Sort();
			sb.Append(sectionTitle).Append(": ");
			foreach ( Patch patch in sortedPatches ) {
				ModContentPack mcp = LoadedModManager.RunningModsListForReading.FirstOrFallback(m => m.assemblies.loadedAssemblies.Contains(patch.PatchMethod.DeclaringType.Assembly));
				sb.Append("[mod: ").Append(mcp.Name).Append("]");
				sb.Append("[harmony-ns: ").Append(patch.owner).Append("]");
				if ( !patch.after.NullOrEmpty() || !patch.before.NullOrEmpty() || patch.priority != Priority.Normal ) {
					sb.Append("[");
					bool addSpace = false;
					if ( !patch.before.NullOrEmpty() ) {
						sb.Append("before: ").Append(patch.before.Join());
						addSpace = true;
					}
					if ( !patch.after.NullOrEmpty() ) {
						if ( addSpace ) sb.Append(" ");
						sb.Append("after: ").Append(patch.after.Join());
						addSpace = true;
					}
					if ( patch.priority != Priority.Normal ) {
						if ( addSpace ) sb.Append(" ");
						sb.Append("priority: ").Append(patch.priority);
					}
					sb.Append("]");
				}
				sb.Append(patch.PatchMethod.DeclaringType.Namespace).Append(".").Append(patch.PatchMethod.DeclaringType.Name).Append(".").Append(patch.PatchMethod.Name).Append(" ");
			}
		}
	}
}
