﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Verse;

namespace RIMMSqol.genericSettings
{
    public class PatchOperation_QoLRemoveUnconfiguredProperties : PatchOperationPathed
    {
        public PatchOperation_QoLRemoveUnconfiguredProperties() { }

        public PatchOperation_QoLRemoveUnconfiguredProperties(string xpath) : this()
        {
            this.xpath = xpath;
        }

        override protected bool ApplyWorker(XmlDocument xml)
        {
            foreach (XmlNode xmlNode in xml.SelectNodes(xpath))
            {
                PurgeNode(xmlNode);
            }
            return true;
        }

        protected void PurgeNode(XmlNode xmlNode)
        {
            //if isConfigured was found, we remove all properties marked as unconfigured if they are present(in case of null values they might not)
            Dictionary<string, XmlNode> childs = new Dictionary<string, XmlNode>();
            foreach ( XmlNode n in xmlNode.ChildNodes )
            {
                childs[n.Name] = n;
            }
            bool hasAnyPropertyConfigured = false;
            bool usesConfigurationStatus = childs.ContainsKey("isConfigured");
            if (usesConfigurationStatus)
            {
                string s = childs["isConfigured"].InnerText;
                string[] parts = s.Split(';');
                int j = parts.Length - 1;
                XmlNode existingPropertyNode = null;
                for (int i = 0; i < j; i += 2)
                {
                    if (parts[i + 1] == "f")
                    {
                        if (childs.TryGetValue(parts[i], out existingPropertyNode))
                        {
                            xmlNode.RemoveChild(existingPropertyNode);
                            if (childs.TryGetValue(parts[i] + "_type", out existingPropertyNode))
                            {
                                xmlNode.RemoveChild(existingPropertyNode);
                            }
                        }
                    }
                    else if (parts[i] != "Version" || childs.ContainsKey("Version") ) hasAnyPropertyConfigured = true;
                }
            }
            else hasAnyPropertyConfigured = true;

            childs.Clear();
            childs = null;
            if (hasAnyPropertyConfigured)
            {
                //we traverse into child nodes that have childnodes of their own.
                foreach (XmlNode child in xmlNode.ChildNodes)
                {
                    if (child.HasChildNodes)
                    {
                        PurgeNode(child);
                    }
                    else
                    {
                        XmlAttributeCollection attrs = child.Attributes;
                        if (attrs != null && attrs.Count > 0 && attrs["IsDefault"] != null)
                        {
                            xmlNode.RemoveChild(child);
                        }
                    }
                }
            }
            else
            {
                //Settings use configuration status but have nothing configured. The entire entry can be removed.
                xmlNode.ParentNode.RemoveChild(xmlNode);
            }
        }
    }
}
