﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 29.08.2019
 * Time: 01:09
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.pathfinding
{
	/// <summary>
	/// Description of Designator_PathfinderMine.
	/// </summary>
	public class Designator_PathfinderMine : Designator
	{
		public override int DraggableDimensions { get { return 2; } }

		public override bool DragDrawMeasurements { get { return true; } }

		public Designator_PathfinderMine() {
			this.soundDragSustain = SoundDefOf.Designate_DragStandard;
			this.soundDragChanged = SoundDefOf.Designate_DragStandard_Changed;
			this.useMouseIcon = true;
			
			this.defaultLabel = "DesignatorPathfinderMine".Translate();
			this.defaultDesc = "DesignatorPathfinderMineDesc".Translate();
			this.icon = Materials.designatorPathfinderMine;
			this.soundSucceeded = SoundDefOf.Designate_ZoneAdd;
			this.hotKey = KeyBindingDefOf.Misc11;
		}

		public override void SelectedUpdate() {
			GenUI.RenderMouseoverBracket();
            MapComponent_PathfinderDirections grid = base.Map.GetComponent<MapComponent_PathfinderDirections>();
            if (grid == null)
            {
                grid = new MapComponent_PathfinderDirections(base.Map);
                base.Map.components.Add(grid);
            }
			grid.MarkForDraw();
		}
		
		public override AcceptanceReport CanDesignateCell(IntVec3 loc) {
			return loc.InBounds(base.Map);
		}

		public override void DesignateMultiCell(IEnumerable<IntVec3> cells) {
			List<IntVec3> allowedCells = null;
			if ( cells != null ) {
				//we don't want to apply the single cell processing for every selected cell, 
				//instead we want to work on the perimeter cells based on their position in the rect.
				DesignationDragger dragger = Find.DesignatorManager.Dragger;
				FieldInfo fi = typeof(DesignationDragger).GetField("startDragCell", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
				IntVec3 startDragCell = (IntVec3)fi.GetValue(dragger);
				
				allowedCells = new List<IntVec3>();
				int minX = int.MaxValue, minZ = int.MaxValue, maxX = int.MinValue, maxZ = int.MinValue;
				bool warned = false;
				foreach ( IntVec3 v in cells ) {
					if (this.CanDesignateCell(v).Accepted) {
						minX = Math.Min(minX,v.x);
						minZ = Math.Min(minZ,v.z);
						maxX = Math.Max(maxX,v.x);
						maxZ = Math.Max(maxZ,v.z);
						allowedCells.Add(v);
						if (!warned) {
							warned = this.ShowWarningForCell(v);
						}
					}
				}
				
				MapComponent_PathfinderDirections grid = base.Map.GetComponent<MapComponent_PathfinderDirections>();
	            if (grid == null) {
	                grid = new MapComponent_PathfinderDirections(base.Map);
	                base.Map.components.Add(grid);
	            }
				
				foreach ( IntVec3 v in allowedCells ) {
					//sides with differentiation between inbetween and caps
					if ( v.x == minX ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x -= 1;
			            if ( v.z != minZ && v.z != maxZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.East, Direction8Way.NorthEast, Direction8Way.SouthEast});
			            else if ( v.z != minZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.East, Direction8Way.SouthEast});
			            else if ( v.z != maxZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.East, Direction8Way.NorthEast});
			            else grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.East});
					}
					if ( v.x == maxX ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x += 1;
			            if ( v.z != minZ && v.z != maxZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.West, Direction8Way.NorthWest, Direction8Way.SouthWest});
			            else if ( v.z != minZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.West, Direction8Way.SouthWest});
			            else if ( v.z != maxZ )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.West, Direction8Way.NorthWest});
			            else grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.West});
					}
					if ( v.z == minZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.z -= 1;
			            if ( v.x != minX && v.x != maxX )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.North, Direction8Way.NorthEast, Direction8Way.NorthWest});
			            else if ( v.x != minX )
							grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.North, Direction8Way.NorthWest});
			            else if ( v.x != maxX )
							grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.North, Direction8Way.NorthEast});
			            else grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.North});
					}
					if ( v.z == maxZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.z += 1;
			            if ( v.x != minX && v.x != maxX )
			            	grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.South, Direction8Way.SouthEast, Direction8Way.SouthWest});
			            else if ( v.x != minX )
							grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.South, Direction8Way.SouthWest});
			            else if ( v.x != maxX )
							grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.South, Direction8Way.SouthEast});
			            else grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.South});
					}
					//corners
					if ( v.x == minX && v.z == minZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x -= 1;
			            adjacentCell.z -= 1;
			            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.NorthEast});
					}
					if ( v.x == maxX && v.z == minZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x += 1;
			            adjacentCell.z -= 1;
			            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.NorthWest});
					}
					if ( v.x == minX && v.z == maxZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x -= 1;
			            adjacentCell.z += 1;
			            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.SouthEast});
					}
					if ( v.x == maxX && v.z == maxZ ) {
						IntVec3 adjacentCell = v;
			            adjacentCell.x += 1;
			            adjacentCell.z += 1;
			            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.SouthWest});
					}
				}
			}
			this.Finalize(!allowedCells.NullOrEmpty());
		}
		
		public override void DesignateSingleCell(IntVec3 c) {
            MapComponent_PathfinderDirections grid = base.Map.GetComponent<MapComponent_PathfinderDirections>();
            if (grid == null) {
                grid = new MapComponent_PathfinderDirections(base.Map);
                base.Map.components.Add(grid);
            }
            
            IntVec3 adjacentCell = c;
            adjacentCell.x = adjacentCell.x-1;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.East});
            adjacentCell.z = adjacentCell.z+1;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.SouthEast});
            adjacentCell.z = adjacentCell.z-2;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.NorthEast});
            
            adjacentCell = c;
            adjacentCell.x = adjacentCell.x+1;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.West});
            adjacentCell.z = adjacentCell.z+1;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.SouthWest});
            adjacentCell.z = adjacentCell.z-2;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.NorthWest});
            
            adjacentCell = c;
            adjacentCell.z += 1;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.South});
            adjacentCell.z -= 2;
            grid.RemoveDirections(adjacentCell, new Direction8Way[]{Direction8Way.North});
		}
		
		public override void RenderHighlight(List<IntVec3> dragCells) {
			float altitude = AltitudeLayer.MetaOverlays.AltitudeFor();
			Vector3 v = new Vector3();
			foreach ( IntVec3 cell in dragCells.Where(c=>CanDesignateCell(c).Accepted) ) {
				v.Set((float)cell.x + 0.5f, altitude, (float)cell.z + 0.5f);
				Graphics.DrawMesh(MeshPool.plane10, v, Quaternion.identity, DesignatorUtility.DragHighlightCellMat, 0);
			}
		}
	}
}
