﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 21.02.2019
 * Time: 04:06
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony
{
	/// <summary>
	/// Description of QOLCityDialog_PawnIndicatorStages.
	/// </summary>
	public class QOLCityDialog_PawnIndicatorStages : Window
	{
		protected readonly static Vector2 portraitSize = new Vector2(70f,70f);
		protected readonly Color colorAffinity = new Color(0.352f, 0.5f, 0.352f), colorSuppression = new Color(0.352f, 0.625f, 0.625f), colorOppression = new Color(1f, 0.5f, 0.352f);
		protected const float gap = 4f;
		protected Vector2 scrollPosition;
		protected QOLCityState state;
		protected QOLSocietyIndicatorDef indicator;
		protected List<Pawn> pawns;
		protected List<List<QOLCityStateSocietyIndicatorStageProperties>> rows;
		protected float cellWidthMin;
		
		override public Vector2 InitialSize {
			get {
				return new Vector2(640f+2*Margin, 300f+2*Margin);
			}
		}
		
		public QOLCityDialog_PawnIndicatorStages(QOLSocietyIndicatorDef indicator, QOLCityState state) {
			this.closeOnCancel = true;
			this.doCloseX = true;
			this.onlyOneOfTypeAllowed = false;
			this.absorbInputAroundWindow = true;
			this.forcePause = true;
			this.indicator = indicator;
			this.state = state;
			
			pawns = new List<Pawn>();
			rows = new List<List<QOLCityStateSocietyIndicatorStageProperties>>();
			foreach ( QOLCityStatePawnProperties pp in state.pawnProperties.Values ) {
				Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> stages = pp.sip[indicator];
				List<QOLCityStateSocietyIndicatorStageProperties> row = new List<QOLCityStateSocietyIndicatorStageProperties>();
				pawns.Add(pp.p);
				foreach ( QOLSocietyIndicatorStage stageDeclared in indicator.stages ) {
					QOLCityStateSocietyIndicatorStageProperties props;
					if ( stages.TryGetValue(stageDeclared, out props) ) {
						row.Add(props);
					} else {
						row.Add(new QOLCityStateSocietyIndicatorStageProperties());
					}
				}
				rows.Add(row);
			}
			this.cellWidthMin = -1;
		}
		
		public override void DoWindowContents(Rect inRect) {
			float ButtonHeight = QOLMod.LineHeight(GameFont.Medium) + 4f;
			float clmnPtrWidth = portraitSize.x+20f;
			float labelPtrHeight = QOLMod.LineHeight(GameFont.Tiny);
			float clmnPtrHeight = labelPtrHeight+portraitSize.y+gap;
			Vector2 offsetPortrait = new Vector2((clmnPtrWidth-portraitSize.x)/2f,0f);
			
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			Color oldColor = GUI.color;
			GUI.color = Color.white;
			
			if ( cellWidthMin < 0f ) {
				cellWidthMin = QOLMod.CalcSize(1.ToString("000%"), GameFont.Medium, TextAnchor.UpperLeft, false).x;
				
				foreach ( QOLSocietyIndicatorStage stageDeclared in indicator.stages ) {
					cellWidthMin = Mathf.Max(cellWidthMin, QOLMod.CalcSize(stageDeclared.label, GameFont.Tiny, TextAnchor.UpperLeft, false).x);
				}
			}
			Text.Anchor = TextAnchor.MiddleCenter;
			Text.Font = GameFont.Tiny;
			Rect cellRect = new Rect(inRect.x + clmnPtrWidth + gap, inRect.y, cellWidthMin, Text.LineHeight);
			foreach ( QOLSocietyIndicatorStage stageDeclared in indicator.stages ) {
				Widgets.Label(cellRect, stageDeclared.label);
				cellRect.x += cellRect.width + gap;
			}
			
			Rect viewRect = new Rect(0,0,inRect.width - QOLMod.VerticalScrollbarWidth(),Mathf.Max(inRect.height - Text.LineHeight, rows.Count*(clmnPtrHeight+gap)));
			
			Widgets.BeginScrollView(new Rect(inRect.x, inRect.y + Text.LineHeight, inRect.width, inRect.height - Text.LineHeight), ref scrollPosition, viewRect);
			try {
				Text.Font = GameFont.Tiny;
				Text.Anchor = TextAnchor.MiddleCenter;
				Rect portraitRect, labelRect;
				portraitRect = new Rect(offsetPortrait.x,offsetPortrait.y,portraitSize.x,portraitSize.y);
				labelRect = new Rect(0,clmnPtrHeight-labelPtrHeight,clmnPtrWidth,labelPtrHeight);
				foreach ( Pawn p in pawns ) {
					if (Mouse.IsOver(portraitRect)||Mouse.IsOver(labelRect)) {
						Widgets.DrawHighlight(portraitRect);
						Widgets.DrawHighlight(labelRect);
					}
					
					RenderTexture tx = PortraitsCache.Get(p, portraitSize, Rot4.South);
					GUI.DrawTexture(portraitRect, tx);
					Widgets.Label(labelRect, p.LabelShort);
					portraitRect.y += clmnPtrHeight + gap;
					labelRect.y += clmnPtrHeight + gap;
				}
				
				Text.Anchor = TextAnchor.MiddleRight;
				Rect cellARect = new Rect(clmnPtrWidth + gap,0f,cellWidthMin,Text.LineHeight);
				Rect cellSRect = new Rect(clmnPtrWidth + gap,Text.LineHeight,cellWidthMin,Text.LineHeight);
				Rect cellORect = new Rect(clmnPtrWidth + gap,Text.LineHeight * 2f,cellWidthMin,Text.LineHeight);
				for ( int row = 0; row < rows.Count; row++ ) {
					List<QOLCityStateSocietyIndicatorStageProperties> cells = rows[row];
					
					for ( int cell = 0; cell < cells.Count; cell++ ) {
						QOLCityStateSocietyIndicatorStageProperties props = cells[cell];
						
						GUI.color = colorAffinity;
						Widgets.Label(cellARect, props.affinity.ToString("000%"));
						GUI.color = colorSuppression;
						Widgets.Label(cellSRect, props.suppression.ToString("000%"));
						GUI.color = colorOppression;
						Widgets.Label(cellORect, props.oppression.ToString("000%"));
						
						cellARect.x += cellWidthMin + gap;
						cellSRect.x += cellWidthMin + gap;
						cellORect.x += cellWidthMin + gap;
					}
					
					cellARect.y += clmnPtrHeight + gap;
					cellSRect.y += clmnPtrHeight + gap;
					cellORect.y += clmnPtrHeight + gap;
					cellARect.x = clmnPtrWidth + gap;
					cellSRect.x = clmnPtrWidth + gap;
					cellORect.x = clmnPtrWidth + gap;
				}
			} finally {
				Widgets.EndScrollView();
				Text.Font = oldFont;
				Text.Anchor = oldAnchor;
				GUI.color = oldColor;
			}
		}
	}
}
