﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 17.11.2020
 * Time: 12:22
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RimWorld;
using RimWorld.Planet;
using Verse;

namespace RIMMSqol.remnantcolony
{
	/// <summary>
	/// Description of WorldObjectCompProperties_CaravanCityInteraction.
	/// </summary>
	public class WorldObjectCompProperties_CaravanCityInteraction : WorldObjectCompProperties
	{
		public WorldObjectCompProperties_CaravanCityInteraction()
		{
			this.compClass = typeof(CaravanCityInteractionComp);
		}
	}
	
	
	public class CaravanCityInteractionComp : WorldObjectComp {
		public override IEnumerable<FloatMenuOption> GetFloatMenuOptions(Caravan caravan) {
			foreach ( FloatMenuOption fmo in CaravanArrivalActionUtility.GetFloatMenuOptions<CaravanArrivalAction_VisitCity>(
				() => CaravanArrivalAction_VisitCity.CanVisit(caravan, parent),() => new CaravanArrivalAction_VisitCity(parent as QOLCity), 
				"VisitCity".Translate(parent.Label), caravan, parent.Tile, parent, null) ) {
				yield return fmo;
			}
			foreach ( FloatMenuOption fmo in CaravanArrivalActionUtility.GetFloatMenuOptions<CaravanArrivalAction_JoinCity>(
				() => CaravanArrivalAction_JoinCity.CanJoin(caravan, parent),() => new CaravanArrivalAction_JoinCity(parent as QOLCity), 
				"JoinCity".Translate(parent.Label), caravan, parent.Tile, parent, null) ) {
				yield return fmo;
			}
		}
		
		public override IEnumerable<Gizmo> GetCaravanGizmos(Caravan caravan) {
			QOLCity city = parent as QOLCity;
			if ( city == null ) {
				Log.Error("Type \"" + this.GetType() + "\" is not usable on non city world objects!");
				yield break;
			}
			if (city.CanTradeNow && parent.Tile == caravan.Tile && city.Faction == caravan.Faction ) {
				Pawn negotiator = BestCaravanPawnUtility.FindBestNegotiator(caravan, city.Faction, city.TraderKind)/* ?? 
					caravan.PawnsListForReading.FirstOrFallback()*/;
				
				Command_Action command_Action = new Command_Action();
				command_Action.defaultLabel = "CommandTrade".Translate();
				command_Action.defaultDesc = "CommandTradeDesc".Translate();
				command_Action.icon = Materials.iconTradeWithCity;
				command_Action.action = delegate { city.TryOpenComms(negotiator); };
				if (negotiator == null) {
					command_Action.Disable("CommandTradeFailNoNegotiator".Translate());
				}
				if (negotiator != null && negotiator.skills.GetSkill(SkillDefOf.Social).TotallyDisabled) {
					command_Action.Disable("CommandTradeFailSocialDisabled".Translate());
				}
				yield return command_Action;
				
				
				Pawn unwantedImmigrant = null;
				foreach ( Pawn p in caravan.PawnsListForReading ) {
					if ( !city.defCity.population.IsImmigrationAllowed(city,p) ) {
						unwantedImmigrant = p;
						break;
					}
				}
				Thing untradebalItem = null;
				List<Thing> allInventoryItems = CaravanInventoryUtility.AllInventoryItems(caravan);
				foreach ( Thing t in allInventoryItems ) {
					if ( !city.TraderKind.WillTrade(t.def) ) {
						untradebalItem = t;
						break;
					}
				}
				command_Action = new Command_Action();
				command_Action.defaultLabel = "CommandJoin".Translate();
				command_Action.defaultDesc = "CommandJoinDesc".Translate();
				command_Action.icon = Materials.iconJoinCity;
				command_Action.action = delegate {
					List<Thing> lst = CaravanInventoryUtility.AllInventoryItems(caravan);
					foreach ( Thing t in lst ) {
						city.GiveSoldThingToTrader(t,t.stackCount,null);
					}
					foreach ( Pawn p in caravan.PawnsListForReading ) {
						city.AddCitizen(p);
						caravan.Notify_PawnRemoved(p);
					}
					caravan.pawns.Clear();
					caravan.Destroy();
				};
				if (unwantedImmigrant != null) {
					command_Action.Disable("CommandJoinFailInvalidImmigrant".Translate(unwantedImmigrant.Named("pawn")));
				} else if (untradebalItem != null) {
					command_Action.Disable("CommandJoinFailInvalidItem".Translate(untradebalItem.Named("item")));
				}
				
				yield return command_Action;
			}
		}
	}
	
	public class CaravanArrivalAction_VisitCity : CaravanArrivalAction {
		private QOLCity city;
		
		public override string Label { get { return "VisitCity".Translate(city.Label); } }

		public override string ReportString { get { return "CaravanVisiting".Translate(city.Label); } }
		
		public CaravanArrivalAction_VisitCity() {
		}
		
		public CaravanArrivalAction_VisitCity(QOLCity city) {
			this.city = city;
		}
		
		public override FloatMenuAcceptanceReport StillValid(Caravan caravan, int destinationTile)
		{
			FloatMenuAcceptanceReport floatMenuAcceptanceReport = base.StillValid(caravan, destinationTile);
			if (!floatMenuAcceptanceReport)
			{
				return floatMenuAcceptanceReport;
			}
			if (city != null && city.Tile != destinationTile)
			{
				return false;
			}
			return CaravanArrivalAction_VisitCity.CanVisit(caravan, city);
		}

		public override void Arrived(Caravan caravan)
		{
			if (city != null && caravan.IsPlayerControlled) {
				Pawn negotiator = BestCaravanPawnUtility.FindBestNegotiator(caravan, city.Faction, city.TraderKind)/* ?? 
					caravan.PawnsListForReading.FirstOrFallback()*/;
				city.TryOpenComms(negotiator);
			}
		}
		
		public static FloatMenuAcceptanceReport CanVisit(Caravan caravan, WorldObject wo) {
			return wo != null && wo.Spawned;
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_References.Look<QOLCity>(ref this.city, "city", false);
		}
	}
	
	public class CaravanArrivalAction_JoinCity : CaravanArrivalAction {
		private QOLCity city;
		
		public override string Label { get { return "VisitCity".Translate(city.Label); } }

		public override string ReportString { get { return "CaravanVisiting".Translate(city.Label); } }
		
		public CaravanArrivalAction_JoinCity() {
		}
		
		public CaravanArrivalAction_JoinCity(QOLCity city) {
			this.city = city;
		}
		
		public override FloatMenuAcceptanceReport StillValid(Caravan caravan, int destinationTile)
		{
			FloatMenuAcceptanceReport floatMenuAcceptanceReport = base.StillValid(caravan, destinationTile);
			if (!floatMenuAcceptanceReport)
			{
				return floatMenuAcceptanceReport;
			}
			if (city != null && city.Tile != destinationTile)
			{
				return false;
			}
			return CaravanArrivalAction_JoinCity.CanJoin(caravan, city);
		}

		public override void Arrived(Caravan caravan)
		{
			if (city != null && caravan.IsPlayerControlled) {
				List<Thing> items = CaravanInventoryUtility.AllInventoryItems(caravan);
				foreach ( Thing t in items ) {
					city.GiveSoldThingToTrader(t,t.stackCount,null);
				}
				foreach ( Pawn p in caravan.PawnsListForReading ) {
					city.AddCitizen(p);
					caravan.Notify_PawnRemoved(p);
				}
				caravan.pawns.Clear();
				caravan.Destroy();
			}
		}
		
		static public FloatMenuAcceptanceReport CanJoin(Caravan caravan, WorldObject wo) {
			if ( wo == null || !wo.Spawned ) return false;
			
			QOLCity city = wo as QOLCity;
			if ( city == null ) return false;
			
			foreach ( Pawn p in caravan.PawnsListForReading ) {
				if ( !city.defCity.population.IsImmigrationAllowed(city,p) ) {
					return FloatMenuAcceptanceReport.WithFailReason("CommandJoinFailInvalidImmigrantReason".Translate(p.Named("pawn")));
				}
			}
			
			List<Thing> lst = CaravanInventoryUtility.AllInventoryItems(caravan);
			foreach ( Thing t in lst ) {
				if ( !city.TraderKind.WillTrade(t.def) ) {
					return FloatMenuAcceptanceReport.WithFailReason("CommandJoinFailInvalidItemReason".Translate(t.Named("item")));
				}
			}
			
			return true;
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_References.Look<QOLCity>(ref this.city, "city", false);
		}
	}
}
