﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 13.12.2018
 * Time: 19:21
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using RIMMSqol.remnantcolony.listeners;
using RIMMSqol.translations;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.actions
{
	public class QOLCityAction_Repair : QOLCityAction {
		protected int workAllocatedInt;
		protected QOLCityActionConfiguration_Repair conf;
		public int WorkAllocated { get {return workAllocatedInt;} }
		
		public QOLCityAction_Repair() : base(null) {}
		
		public QOLCityAction_Repair(QOLCityActionDef def) : base(def) {
			conf = def.action as QOLCityActionConfiguration_Repair;
			if ( conf == null ) {
				Log.Error("No matching conf found in "+typeof(QOLCityAction_Repair).Name+"!");
			}
			workAllocatedInt = 0;
		}

		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<int>(ref workAllocatedInt, "workAllocated", 0);
			if (Scribe.mode == LoadSaveMode.ResolvingCrossRefs) {
				if ( def != null ) {
					conf = def.action as QOLCityActionConfiguration_Repair;
					if ( conf == null ) {
						Log.Error("No matching conf found in "+typeof(QOLCityAction_Repair).Name+"!");
					}
				} else {
					Log.Error("No definition found in "+typeof(QOLCityAction_Repair).Name+"!");
				}
			}
		}
		
		public override void OnClick(QOLCity city, Pawn negotiator) {
			Find.WindowStack.Add(new Dialog_Spinner(workAllocatedInt, 
			                                        newValue => newValue >= 0 && newValue <= conf.MaxCyclePerInstance * instances, 
			                                        newValue => { 
			                                        	workAllocatedInt = newValue;
														city.Refresh();			                                        	
			                                        }, 1, def.label, def.description));
		}
	}
	
	public class QOLCityActionConfiguration_Repair : QOLCityListener_ConversionConfiguration_ActionBase {
		public List<ThingCategoryDef> categories;
		public float hitPointMultiplier = 1f;
		//public int maxLabourPerInstance = 1;
		public override int ActiveInstances(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state) {
			QOLCityActionDef action = ((QOLCityAction_Convert)owner).def;
			return city.projects.Count(p=>p.IsActive && p.def.actions.Contains(action)); 
		}
		public override int CyclesAllocated(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state) {
			QOLCityAction_Repair action = (QOLCityAction_Repair)owner;
			int workEffectively = Math.Max(0,Math.Min(action.WorkAllocated,Mathf.FloorToInt(city.LabourBalance(state))));
			//if not all repairs are needed we can reduce the labour drain for next turn
			if ( workEffectively > 0 ) {
				List<ThingCategoryDef> tc = AllCategories;
				int missingHealthInTotal = city.GetDirectlyHeldThings().Where(thing=>thing.HitPoints < thing.MaxHitPoints && thing.def.thingCategories != null && tc.SharesElementWith(thing.def.thingCategories)).Sum(thing=>thing.MaxHitPoints-thing.HitPoints);
				workEffectively = Math.Min(workEffectively,Mathf.CeilToInt(missingHealthInTotal / hitPointMultiplier));
			}
			return workEffectively;
		}
		public override void CyclesPerformed(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state, int cyclesPerformed) {
			int workEffectively = Mathf.RoundToInt(cyclesPerformed * hitPointMultiplier);
			QOLCityAction_Repair action = (QOLCityAction_Repair)owner;
			if ( !categories.NullOrEmpty() ) {
				List<ThingCategoryDef> tc = AllCategories;
				if ( workEffectively > 0 ) {
					int hitPointsToRepair = Mathf.RoundToInt(workEffectively * hitPointMultiplier);
					while ( hitPointsToRepair > 0 ) {
						Thing t;
						if ( city.GetDirectlyHeldThings().Where(thing=>
						                                        thing.HitPoints < thing.MaxHitPoints &&
						                                        thing.def.thingCategories != null &&
						                                        tc.SharesElementWith(thing.def.thingCategories)).TryRandomElement(out t) ) {
							int missingHitpoints = t.MaxHitPoints - t.HitPoints;
							if ( hitPointsToRepair <= missingHitpoints ) {
								t.HitPoints += hitPointsToRepair;
							} else {
								t.HitPoints = t.MaxHitPoints;
							}
							hitPointsToRepair -= missingHitpoints;
						} else {
							break;
						}
					}
				}
				workEffectively = 0;
				//Clearing the dead mans apparel flag on all things that are fully repaired
				foreach ( Thing t in city.GetDirectlyHeldThings().Where(thing=>thing.HitPoints >= thing.MaxHitPoints && IsDeadmansApparel(thing) && thing.def.thingCategories != null &&
				                                                        tc.SharesElementWith(thing.def.thingCategories)) ) {
					try {
						typeof(Apparel).GetField("wornByCorpseInt", BindingFlags.Instance|BindingFlags.NonPublic).SetValue(t,false);
					} catch {
						Log.Error("Failed to clear dead man status of apparel \""+t.def.defName+"\".");
					}
				}
			} else {
				Log.Error("No thingCategories defined in action segment of def "+action.def.defName+"!");
			}
		}
		
		protected bool IsDeadmansApparel(Thing t) {
			if ( t.def.IsApparel && t.def.apparel.careIfWornByCorpse ) {
				Apparel apparel = t as Apparel;
				return apparel != null && apparel.WornByCorpse;
			}
			return false;
		}
		
		public override QOLCityAction CreateAction(QOLCityActionDef def) {
			return new QOLCityAction_Repair(def);
		}
		
		public List<ThingCategoryDef> AllCategories {
			get {
				if ( categories != null ) return (from cs in categories from c in cs.ThisAndChildCategoryDefs select c).Distinct().ToList();
				return new List<ThingCategoryDef>();
			}
		}
	}
}
