﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 21.11.2018
 * Time: 04:01
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Text;
using Verse;

namespace RIMMSqol.remnantcolony.callbacks
{
	/// <summary>
	/// Description of QOLCityCallback_ResourceBooster.
	/// </summary>
	public class QOLCityCallback_ResourceBooster : QOLCityCallback
	{
		/*
		 * TODO Use something like IngredientValueGetter to allow amounts that refer to something else than item count
		 * TODO Extended filter settings
		 */
		public ThingDef thing;
		public ThingCategoryDef thingCategory;
		public QOLCityProjectDef boosterProject;
		public int amount;
		
		public void Callback(Def owner, QOLCity city, Pawn negotiator) {
			QOLCityResourceDef resource;
			if ( (resource = owner as QOLCityResourceDef) != null ) {
				if ( city != null ) {
					if ( !city.defCity.projects.Contains(boosterProject) ) {
						Log.Warning("Booster project "+boosterProject.defName+" is not allowed by city configuration "+city.defCity.defName+".");
						return;
					}
					if ( city.GetDirectlyHeldThings() != null ) {
						if ( HasAtLeast(city.GetDirectlyHeldThings()) ) {
							StringBuilder sb = new StringBuilder();
							sb.Append("Spent ").Append(amount).Append(" items of the following type(s) to gain 1 ").Append(resource.label).Append(" core for ").Append(boosterProject.lifetime).AppendLine(" turns?");
							if ( thing != null ) {
								sb.AppendLine(thing.label);
							} else if ( thingCategory != null ) {
								foreach ( ThingCategoryDef d in thingCategory.ThisAndChildCategoryDefs ) {
									sb.AppendLine(d.label);
								}
							} else {
								sb.AppendLine("---");
							}
							Find.WindowStack.Add(Dialog_MessageBox.CreateConfirmation(sb.ToString(), delegate {
                              	if ( RemoveFromOwner(city.GetDirectlyHeldThings()) ) {
                              		city.AddProjectAndRefresh(boosterProject);
	                          	} else {
	                      			Log.Error("Failed to remove the resources for triggering the booster!");
	                          	}
							}, false, null));
						} else {
							StringBuilder sb = new StringBuilder();
							sb.Append("Not enough permissable items to spent on gaining 1 ").Append(resource.label).Append(" core for ").Append(boosterProject.lifetime).AppendLine(" turns.");
							sb.Append("You need ").Append(amount).Append(" items of the following type(s).");
							if ( thing != null ) {
								sb.AppendLine(thing.label);
							} else if ( thingCategory != null ) {
								foreach ( ThingCategoryDef d in thingCategory.ThisAndChildCategoryDefs ) {
									sb.AppendLine(d.label);
								}
							} else {
								sb.AppendLine("---");
							}
							Find.WindowStack.Add(new Dialog_MessageBox(sb.ToString(), null, null, null, null, null, true, null, null));
						}
					} else {
						Log.Error("City's GetDirectlyHeldThings returned a null owner in callback "+this.GetType().Name+"!");
					}
				} else {
					Log.Error("City was null in "+this.GetType().Name+" callback!");
				}
			} else {
				Log.Error("Tried to use resource callback "+this.GetType().Name+" for defintion "+owner+" which isn't a resource!");
			}
		}
		
		protected Func<Thing,bool> ThingFilter() {
			Func<Thing,bool> isAllowed;
			if ( thing != null ) {
				isAllowed = t => t.def == thing;
			} else if ( thingCategory != null ) {
				List<ThingCategoryDef> tc = new List<ThingCategoryDef>(thingCategory.ThisAndChildCategoryDefs);
				isAllowed = t => t.def.thingCategories != null && tc.SharesElementWith(t.def.thingCategories);
			} else {
				return t => false;
			}
			return isAllowed;
		}
		
		protected bool HasAtLeast(ThingOwner owner) {
			Func<Thing,bool> isAllowed = ThingFilter();
			int countSoFar = 0;
			return owner.FirstOrFallback((Thing t) => {
	          	if ( isAllowed(t) ) countSoFar += t.stackCount;
	          	return countSoFar >= amount;
	          }, null) != null;
		}
		
		protected bool RemoveFromOwner(ThingOwner owner) {
			if ( amount <= 0 ) return true;
			
			if ( !HasAtLeast(owner) ) {
				return false;
			}
			
			int count = amount;
			Func<Thing,bool> isAllowed = ThingFilter();
			Thing thingToSplit = null;
			owner.RemoveAll(t=>{ 
                	if ( isAllowed(t) ) {
                		if ( count >= t.stackCount ) {
                			count -= t.stackCount; 
    						return true;
                		}
                		thingToSplit = t;
                	}
                	return false;
                });
			if ( count > 0 ) {
				if ( thingToSplit != null ) {
					Thing thingRemoved = owner.Take(thingToSplit,count);
					return thingRemoved != null && thingRemoved.stackCount == count;
				}
			} else {
				return true;
			}
			return false;
		}
	}
}
