﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 06.02.2019
 * Time: 04:50
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RIMMSqol.remnantcolony.actions;
using RIMMSqol.remnantcolony.projects;
using RIMMSqol.renderers;
using RIMMSqol.translations;
using RIMMSqol.extensions;
using UnityEngine;
using Verse;
using RimWorld;

namespace RIMMSqol.remnantcolony.cities
{
	/// <summary>
	/// Description of QOLCity_Default.
	/// </summary>
	public class QOLCity_Default : QOLCity
	{
		override public void TryOpenComms(Pawn negotiator) {
			base.TryOpenComms(negotiator);
			
			Dictionary<string, PageRenderer> pages = new Dictionary<string, PageRenderer>();
			string title = "overview";
			title.TryTranslate(out title);
			PageRenderer prOverview = new PageRenderer(f=>title+" "+GetCallLabel(), null, null, null);
			prOverview.AddChild(
				new VerticalSplitRenderer(
					new HorizontalSplitRenderer(
						new QOLCityActionsRenderer(),
						new QOLCityStatsRenderer()
					),
					new HorizontalSplitRenderer(
						new QOLCityProjectRenderer(),
						new QOLCityBuildQueueRenderer()
					)
				)
			);
			
			pages.Add("overview",prOverview);
			
			Flow flow = new Flow(pages,"overview");
			
			flow.flowScope["city"] = this;
			flow.flowScope["negotiator"] = negotiator;
			//keep size down for 1280x768
			Find.WindowStack.Add(new Dialog_Flow(flow,Mathf.Min(Mathf.Ceil(Screen.width * 0.9f), 1500f),Mathf.Min(Mathf.Ceil(Screen.height * 0.9f),850f)));
		}
	}
	
	public class QOLCityActionsRenderer : ParentRendererWithIteration {
		public const float iconSize = 32f;
		public const float gapHorizontal = 4f, gapVertical = 4f;
		protected float lastHeight = iconSize;
		
		override public float getHeight() {
			return lastHeight;
		}
		override public float getWidth() {
			return 500f;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			QOLCity city = (QOLCity)flow.flowScope["city"];
			Pawn negotiator = (Pawn)flow.flowScope["negotiator"];
			
			if ( city.actions.NullOrEmpty() ) {
				lastHeight = 0;
			} else {
				beginIteration(flow);
				try {
					Rect nextRect = new Rect(inRect.x,inRect.y,iconSize,iconSize);
					foreach ( QOLCityAction action in city.actions ) {
						advanceIteration(action);
						
						TooltipHandler.TipRegion(nextRect,()=>action.def.label+" x "+action.instances.ToString("0")+"\n\n"+action.def.description,1);
						if ( Widgets.ButtonImage(nextRect,action.def.Icon) ) {
							action.OnClick(city, negotiator);
						}
						
						nextRect.x += gapHorizontal + nextRect.width;
						if ( nextRect.x > inRect.xMax ) {
							nextRect.x = inRect.x;
							nextRect.y += gapVertical + nextRect.height;
						}
					}
					lastHeight = nextRect.yMax - inRect.yMin;
				} finally {
					endIteration(flow);
				}
			}
		}
	}
	
	public class QOLCityBuildQueueRenderer : ParentRendererWithIteration {
		protected const GameFont font = GameFont.Medium;
		protected const float gapHorizontal = 4f, gapVertical = 4f, lineThickness = 2f;
		static protected string labelTitle = "Queue";
		static protected Vector2 sizeTitle;
		static protected bool initialized = false;
		protected float lastHeight = 0;
		public QOLCityBuildQueueRenderer() {
			AddChild(new QOLCityBuildQueuesRenderer(f=>(QOLCityProject)((IterationItem)f.pageScope["curItem"]).curItem));
		}
		override public float getHeight() {
			return lastHeight;
		}
		override public float getWidth() {
			return 500f;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			if ( !initialized ) {
				labelTitle.TryTranslate(out labelTitle);
				sizeTitle = QOLMod.CalcSize(labelTitle, font);
				initialized = true;
			}
			
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			QOLCity city = (QOLCity)flow.flowScope["city"];
			float lineHeight = QOLMod.LineHeight(font);
			
			GUI.DrawTexture(new Rect(inRect.x, inRect.y, lineThickness, lastHeight),BaseContent.WhiteTex);
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			GUI.Label(new Rect(inRect.x + (gapHorizontal + lineThickness), inRect.y, sizeTitle.x, lineHeight),labelTitle,Text.CurFontStyle);
			
			GUI.DrawTexture(new Rect(inRect.x + (sizeTitle.x + gapHorizontal * 2 + lineThickness), inRect.y, lineThickness, lineHeight),BaseContent.WhiteTex);
			GUI.DrawTexture(new Rect(inRect.x, inRect.y + lineHeight, inRect.width, lineThickness),BaseContent.WhiteTex);
			
			float usedHeight = lineHeight + lineThickness;
			
			beginIteration(flow);
			try {
				BaseRenderer renderer = childs[0];
				Rect nextRect = new Rect(inRect.x + lineThickness + gapHorizontal,inRect.y + usedHeight,inRect.width - 2 * (lineThickness + gapHorizontal),renderer.getHeight());
				foreach ( QOLCityProject project in city.BuildQueue ) {
					advanceIteration(project);
					renderer.DoComponentContents(nextRect, flow);
					nextRect.y += gapVertical + nextRect.height;
				}
				usedHeight = nextRect.yMax - inRect.yMin;
			} finally {
				endIteration(flow);
			}
			
			lastHeight = usedHeight;
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityBuildQueuesRenderer : BaseRenderer {
		protected const float gapHorizontal = 4f;
		protected const float lineThickness = 1f;
		protected const float iconSize = 16f;
		protected const GameFont font = GameFont.Small;
		protected readonly Color iconDeconstructColor = new Color(1f, 0.5f, 0.352f), iconActivateColor = new Color(0.352f, 0.5f, 0.352f),
			iconDeactivateColor = new Color(1f, 0.5f, 0.352f);
		protected float[] clmnSizes = {150f,60f,60f};
		static protected float height = -1;
		protected Func<Flow,QOLCityProject> projectLookup;
		public QOLCityBuildQueuesRenderer(Func<Flow,QOLCityProject> projectLookup) {
			this.projectLookup = projectLookup;
		}
		override public float getHeight() {
			if ( height < 0f ) {
				height = Mathf.Max(QOLMod.LineHeight(font),iconSize);
			}
			return height;
		}
		override public float getWidth() {
			return -1;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			QOLCityProject project = projectLookup(flow);
			
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			float lineHeight = QOLMod.LineHeight(font), height = getHeight();
			Rect nextClmnText = new Rect(inRect.x + gapHorizontal, inRect.y+(getHeight()-lineHeight)/2f, clmnSizes[0], lineHeight);
			Rect nextClmn = new Rect(inRect.x + gapHorizontal, inRect.y, clmnSizes[0], height);
			
			bool wasCapped;
			string labelCapped = QOLMod.CapString(project.def.label,nextClmnText.width,"...",out wasCapped,Text.Font);			
			GUI.Label(nextClmnText, labelCapped, Text.CurFontStyle);
			
			if ( wasCapped && Mouse.IsOver(nextClmnText) ) {
				TooltipHandler.TipRegion(nextClmnText, ()=>project.def.label, 1);
			}
			
			Text.Anchor = TextAnchor.MiddleCenter;
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[1];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[1];
			QOLCity city = (QOLCity)flow.flowScope["city"];
			float labourLeft = project.def.labour - project.labourInvested;
			if ( labourLeft > 0f ) {
				GUI.Label(nextClmnText, ""+((int)labourLeft),Text.CurFontStyle);
			} else {
				GUI.Label(nextClmnText, "---",Text.CurFontStyle);
			}
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[2];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[2];
			
			Rect iconRect = new Rect(nextClmn.x, nextClmn.y+(height-iconSize)/2f, iconSize, iconSize);
			if ( project.def.cancellable ) {
				if ( Widgets.ButtonImage(iconRect, Materials.iconDeconstruct16, iconDeconstructColor) ) {
					flow.addPostRenderCallback(f=>city.RemoveProject(project));
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("CancelProjectBuild".Translate(),1));
				}
			}
			iconRect.x += iconSize + gapHorizontal;
			
			if ( project.IsActive ) {
				if ( project.def.toggleable ) {
					if ( Widgets.ButtonImage(iconRect, Materials.iconPower16, iconDeactivateColor) ) {
						city.DeactivateProject(project);
					}
					GUI.color = Color.white;
					if ( Mouse.IsOver(iconRect) ) {
						TooltipHandler.TipRegion(iconRect, new TipSignal("PauseProjectBuild".Translate(),1));
					}
				}
			} else {
				if ( Widgets.ButtonImage(iconRect, Materials.iconPower16, iconActivateColor) ) {
					city.ActivateProject(project);
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("ResumeProjectBuild".Translate(),1));
				}
			}
			iconRect.x += iconSize + gapHorizontal;
		}
	}
	
	public class QOLCityProjectRenderer : ParentRendererWithIteration {
		protected float lastHeight;
		protected Dictionary<QOLCityProjectCategoryDef,BaseRenderer> renderers;
		protected List<QOLCityProjectCategoryDef> usedCats;
		protected string cachedCity;
		protected const GameFont font = GameFont.Medium;
		protected const GameFont fontMinor = GameFont.Small;
		protected const float gapHorizontal = 4f, gapVertical = 4f, lineThickness = 2f;
		static protected string labelTitle = "Projects";
		static protected Vector2 sizeTitle;
		static protected bool initialized = false;
		
		public QOLCityProjectRenderer() {
			lastHeight = 0;
			renderers = null;
			usedCats = null;
			cachedCity = null;
		}
		override public float getHeight() {
			return lastHeight;
		}
		override public float getWidth() {
			return 500f;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			QOLCity city = (QOLCity)flow.flowScope["city"];
			float lineHeight = QOLMod.LineHeight(font);
			float lineHeightMinor = QOLMod.LineHeight(fontMinor);
			float usedHeight = 0;
			
			if ( !initialized ) {
				labelTitle.TryTranslate(out labelTitle);
				sizeTitle = QOLMod.CalcSize(labelTitle, font);
				initialized = true;
			}
			
			if ( cachedCity != city.GetUniqueLoadID() ) {
				usedCats = new List<QOLCityProjectCategoryDef>(city.defCity.projects.Where(pd=>pd.category != null).Select(pd=>pd.category).Distinct().OrderBy(c=>c));
				renderers = new Dictionary<QOLCityProjectCategoryDef, BaseRenderer>();
				Func<Flow,Pair<QOLCityProjectDef, List<QOLCityProject>>> projectLookup = f=>(Pair<QOLCityProjectDef, List<QOLCityProject>>)((IterationItem)f.pageScope["curItem"]).curItem;
				foreach ( QOLCityProjectCategoryDef d in usedCats ) {
					renderers.Add(d,(BaseRenderer)Activator.CreateInstance(d.renderer,new object[]{projectLookup}));
				}
				cachedCity = city.GetUniqueLoadID();
			}
			
			GUI.DrawTexture(new Rect(inRect.x, inRect.y, lineThickness, lastHeight),BaseContent.WhiteTex);
			Text.Font = font;
			GUI.Label(new Rect(inRect.x + (gapHorizontal + lineThickness), inRect.y, sizeTitle.x, lineHeight),labelTitle,Text.CurFontStyle);
			GUI.DrawTexture(new Rect(inRect.x + (sizeTitle.x + gapHorizontal * 2 + lineThickness), inRect.y, lineThickness, lineHeight),BaseContent.WhiteTex);
			
			Text.Font = fontMinor;
			float yMinor = inRect.y + lineHeight - lineHeightMinor;
			GUI.DrawTexture(new Rect(inRect.x, inRect.y + lineHeight, inRect.width, lineThickness),BaseContent.WhiteTex);
			usedHeight = lineHeight + lineThickness;
			
			Dictionary<QOLCityProjectDef,List<QOLCityProject>> mappedProjects = new Dictionary<QOLCityProjectDef, List<QOLCityProject>>();
			foreach ( QOLCityProject proj in city.projects ) {
				List<QOLCityProject> lst;
				if ( !mappedProjects.TryGetValue(proj.def,out lst) ) {
					lst = new List<QOLCityProject>();
					mappedProjects.Add(proj.def, lst);
				}
				lst.Add(proj);
			}
			foreach ( QOLCityProjectDef proj in city.defCity.projects ) {
				if ( !mappedProjects.ContainsKey(proj) ) {
					mappedProjects.Add(proj,new List<QOLCityProject>());
				}
			}
			
			List<QOLCityProjectDef> tempProjDefs = new List<QOLCityProjectDef>(city.defCity.projects);
			foreach ( QOLCityProjectCategoryDef cat in usedCats ) {
				beginIteration(flow);
				try {
					Vector2 sizeLabel = QOLMod.CalcSize(cat.LabelCap, Text.Font);
					GUI.Label(new Rect(inRect.x + lineThickness + gapHorizontal, inRect.y + usedHeight, sizeLabel.x, lineHeightMinor),cat.LabelCap,Text.CurFontStyle);
					GUI.DrawTexture(new Rect(inRect.x, inRect.y + usedHeight + lineHeightMinor, sizeLabel.x + gapHorizontal * 2, lineThickness),BaseContent.WhiteTex);
					usedHeight += lineHeightMinor + lineThickness + gapVertical;

					BaseRenderer renderer = renderers[cat];
					Rect nextRect = new Rect(inRect.x + lineThickness + gapHorizontal,inRect.y + usedHeight,inRect.width - 2 * (lineThickness + gapHorizontal),renderer.getHeight());
					
					tempProjDefs.RemoveAll(p=>{
					                       	if ( p.category != cat ) return false;
											advanceIteration(new Pair<QOLCityProjectDef, List<QOLCityProject>>(p,mappedProjects[p]));
											renderer.DoComponentContents(nextRect, flow);
											nextRect.y += gapVertical + nextRect.height;
											return true;
					                       });
					usedHeight = nextRect.yMax - inRect.yMin;
				} finally {
					endIteration(flow);
				}
			}
			
			lastHeight = usedHeight;
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityProjectsRenderer : BaseRenderer {
		protected const float gapHorizontal = 4f;
		protected const float lineThickness = 1f;
		protected const float iconSize = 16f, headerIconSize = 16f;
		protected const GameFont font = GameFont.Small;
		protected readonly Color iconDeconstructColor = new Color(1f, 0.5f, 0.352f), iconConstructColor = new Color(0.352f, 0.5f, 0.352f);
		protected float[] clmnSizes = {150f,40f,60f,60f,20f,60f};
		static protected float height = -1;
		protected Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup;
		public QOLCityProjectsRenderer(Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup) {
			this.projectLookup = projectLookup;
		}
		override public float getHeight() {
			if ( height < 0f ) {
				height = Mathf.Max(QOLMod.LineHeight(font),iconSize);
			}
			return height;
		}
		override public float getWidth() {
			return -1;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			GUI.color = Color.white;
			
			//Draw header
			if ( ((IterationItem)flow.pageScope["curItem"]).index == 0 ) {
				float[] hdrClmnOffsets = new float[clmnSizes.Length];
				for ( int i = 0; i < clmnSizes.Length; i++ ) {
					hdrClmnOffsets[i] = inRect.x + gapHorizontal;
					for ( int j = 0; j < i; j++ ) {
						hdrClmnOffsets[i] += clmnSizes[j] + gapHorizontal * 2 + lineThickness;
					}
					hdrClmnOffsets[i] += clmnSizes[i] / 2f - headerIconSize / 2f;
				}
				
				Rect nextHdrIcon = new Rect(hdrClmnOffsets[1], inRect.y - 10f - headerIconSize, headerIconSize, headerIconSize);
				GUI.DrawTexture(nextHdrIcon,Materials.iconHash16);
				if ( Mouse.IsOver(nextHdrIcon) ) {
					TooltipHandler.TipRegion(nextHdrIcon, new TipSignal("NumberOfProjectInstances".Translate(), 1));
				}
				
				nextHdrIcon.x = hdrClmnOffsets[2];
				GUI.DrawTexture(nextHdrIcon,Materials.iconHammer16);
				if ( Mouse.IsOver(nextHdrIcon) ) {
					TooltipHandler.TipRegion(nextHdrIcon, new TipSignal("LabourNeededForProject".Translate(), 1));
				}
				
				nextHdrIcon.x = hdrClmnOffsets[3];
				GUI.DrawTexture(nextHdrIcon,Materials.iconHourglass16);
				if ( Mouse.IsOver(nextHdrIcon) ) {
					TooltipHandler.TipRegion(nextHdrIcon, new TipSignal("ShortestTimeoutForProjectInstance".Translate(), 1));
				}
			}
			
			Pair<QOLCityProjectDef,List<QOLCityProject>> p = projectLookup(flow);
			QOLCityProjectDef project = p.First;
			List<QOLCityProject> projectInstances = p.Second;
			
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			float lineHeight = QOLMod.LineHeight(font), height = getHeight();
			Rect nextClmnText = new Rect(inRect.x + gapHorizontal, inRect.y+(getHeight()-lineHeight)/2f, clmnSizes[0], lineHeight);
			Rect nextClmn = new Rect(inRect.x + gapHorizontal, inRect.y, clmnSizes[0], height);
			
			bool wasCapped;
			string labelCapped = QOLMod.CapString(project.label,nextClmnText.width,"...",out wasCapped,Text.Font);			
			GUI.Label(nextClmnText, labelCapped, Text.CurFontStyle);
			
			if ( Mouse.IsOver(nextClmnText) ) {
				TooltipHandler.TipRegion(nextClmnText, project.Tooltip, 1);
			}
			
			Text.Anchor = TextAnchor.MiddleCenter;
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[1];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[1];
			if ( projectInstances.Any(pin=>!pin.IsActive) ) {
				GUI.color = Color.red;
				if ( Mouse.IsOver(nextClmnText) ) {
					TooltipHandler.TipRegion(nextClmnText, ()=>projectInstances.Count(pin=>pin.IsActive).ToString("00")+"/"+project.maximumInstances.ToString("00"), 1);
				}
			}
			if ( project.maximumInstances > 0 ) 
				GUI.Label(nextClmnText, projectInstances.Count.ToString("00")+"/"+project.maximumInstances.ToString("00"), Text.CurFontStyle);
			else GUI.Label(nextClmnText, projectInstances.Count.ToString("00"), Text.CurFontStyle);
			GUI.color = Color.white;
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[2];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[2];
			
			QOLCity city = (QOLCity)flow.flowScope["city"];
			int years, quadrums, days; float hours;
			//In a city labour is a need. The labour cost of a new project goes to the need and the current value satisfies the currently build projects labour needs.			
			/*
			float labourProvidedPerTick = city.LabourBalance() / 60000f;
			if ( labourProvidedPerTick > 0f ) {
				((long)(project.labour/labourProvidedPerTick)).TicksToPeriod(out years,out quadrums,out days,out hours);
				GUI.Label(nextClmnText, years+"y"+quadrums+"q"+days+"d", Text.CurFontStyle);
			} else {
				GUI.Label(nextClmnText, "---", Text.CurFontStyle);
			}*/
			if ( project.labour > 0f ) {
				GUI.Label(nextClmnText, project.labour.ToString("0.00"), Text.CurFontStyle);
			} else {
				GUI.Label(nextClmnText, "---", Text.CurFontStyle);
			}
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[3];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[3];
			int lifetime = project.lifetime;
			foreach ( QOLCityProject proj in projectInstances ) {
				lifetime = Math.Min(lifetime,Math.Min(0,proj.def.lifetime-proj.turnsActive));
			}
			if ( lifetime > 0 ) {
				//write out when the next instance times out
				((long)lifetime*city.simulationInterval).TicksToPeriod(out years,out quadrums,out days,out hours);
				GUI.Label(nextClmnText, years+"y"+quadrums+"q"+days+"d", Text.CurFontStyle);
			} else {
				GUI.Label(nextClmnText, "---", Text.CurFontStyle);
			}
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[4];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[4];
			Rect iconRect = new Rect(nextClmn.x+(nextClmn.width-iconSize)/2f, nextClmn.y+(height-iconSize)/2f, iconSize, iconSize);
			if ( city.CanBuildProject(project) ) GUI.DrawTexture(iconRect, Widgets.CheckboxOnTex);
			else GUI.DrawTexture(iconRect, Widgets.CheckboxOffTex);
			
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[5];
			nextClmnText.x = nextClmn.x; nextClmnText.width = clmnSizes[5];
			iconRect = new Rect(nextClmn.x, nextClmn.y+(height-iconSize)/2f, iconSize, iconSize);
			if ( city.CanAddProject(project) ) {
				if ( Widgets.ButtonImage(iconRect, Materials.iconConstruct16, iconConstructColor) ) {
					//add to queue, possibly after picking a tile on a worldmap selection.
					if ( project.worldObject != null ) {
						Find.WindowStack.Add(new Dialog_SelectWorldTile(city.Tile,7,"Pick unoccupied location for new district", 
						                                                (i,t)=> {
						                                                	if ( !t.biome.impassable && !Find.WorldObjects.AnyWorldObjectAt(i) ) {
						                                                		city.AddProjectAndRefresh(project,i);
						                                                		return true;
						                                                	}
						                                                	return false;
						                                                }));
					} else {
						city.AddProjectAndRefresh(project);
					}
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("QueueProjectInstance".Translate(),1));
				}
			}
			iconRect.x += iconRect.width + gapHorizontal;
			
			if ( projectInstances.Any(pin=>pin.IsActive) ) {
				if ( project.toggleable && Widgets.ButtonImage(iconRect, Materials.iconPower16, iconDeconstructColor) ) {
					if ( project.worldObject == null ) {
						city.DeactivateProject(projectInstances.First(pin=>pin.IsActive));
					} else {
						Find.WindowStack.Add(new Dialog_Select<QOLCityProject>(pin=>{city.DeactivateProject(pin); return true;},
	                       projectInstances.Where(pin=>pin.IsActive),
	                       pin=>pin.def.label+" "+(pin.worldObject!=null?pin.worldObject.Tile.ToString():"-")));
					}
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("DeactivateProjectInstance".Translate(),1));
				}
			}
			iconRect.x += iconRect.width + gapHorizontal;
			
			if ( projectInstances.Any(pin=>!pin.IsActive) ) {
				if ( Widgets.ButtonImage(iconRect, Materials.iconPower16, iconConstructColor) ) {
					if ( project.worldObject == null ) {
						city.ActivateProject(projectInstances.First(pin=>!pin.IsActive));
					} else {
						Find.WindowStack.Add(new Dialog_Select<QOLCityProject>(pin=>{city.ActivateProject(pin); return true;},
	                       projectInstances.Where(pin=>!pin.IsActive),
	                       pin=>pin.def.label+" "+(pin.worldObject!=null?pin.worldObject.Tile.ToString():"-")));
					}
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("ActivateProjectInstance".Translate(),1));
				}
			}
			iconRect.x += iconRect.width + gapHorizontal;
			
			if ( project.cancellable && projectInstances.Count > 0 ) {
				if ( Widgets.ButtonImage(iconRect, Materials.iconDeconstruct16, iconDeconstructColor) ) {
					//cancel after picking instance
					if ( projectInstances.Count > 1 && project.worldObject != null ) {
						Find.WindowStack.Add(new Dialog_Select<QOLCityProject>(pin=>{flow.addPostRenderCallback(f=>city.RemoveProject(pin)); return true;},projectInstances,pin=>pin.def.label+" "+(pin.worldObject!=null?pin.worldObject.Tile.ToString():"-")));
					} else {
						flow.addPostRenderCallback(f=>city.RemoveProject(projectInstances.First()));
					}
				}
				GUI.color = Color.white;
				if ( Mouse.IsOver(iconRect) ) {
					TooltipHandler.TipRegion(iconRect, new TipSignal("DestroyProjectInstance".Translate(),1));
				}
			}
			
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityCrisisRenderer : BaseRenderer {
		protected const float gapHorizontal = 4f;
		protected const float lineThickness = 1f;
		protected const float iconSize = 16f;
		protected const GameFont font = GameFont.Small;
		protected float[] clmnSizes = {150f,200f};
		static protected float height = -1;
		protected Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup;
		public QOLCityCrisisRenderer(Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup) {
			this.projectLookup = projectLookup;
		}
		override public float getHeight() {
			if ( height < 0f ) {
				height = Mathf.Max(QOLMod.LineHeight(font),iconSize);
			}
			return height;
		}
		override public float getWidth() {
			return -1;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			GUI.color = Color.white;
			
			Pair<QOLCityProjectDef,List<QOLCityProject>> p = projectLookup(flow);
			QOLCityProjectDef project = p.First;
			List<QOLCityProject> projectInstances = p.Second;
			
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			float lineHeight = QOLMod.LineHeight(font), height = getHeight();
			Rect nextClmnText = new Rect(inRect.x + gapHorizontal, inRect.y+(getHeight()-lineHeight)/2f, clmnSizes[0], lineHeight);
			Rect nextClmn = new Rect(inRect.x + gapHorizontal, inRect.y, clmnSizes[0], height);
			GUI.Label(nextClmnText, project.label, Text.CurFontStyle);
			
			//Tooltip
			if ( Mouse.IsOver(nextClmnText) ) {
				TooltipHandler.TipRegion(nextClmnText, project.Tooltip, 1);
			}
			
			//Progressbar
			GUI.DrawTexture(new Rect(nextClmn.x+nextClmn.width+gapHorizontal, nextClmn.y, lineThickness, height),BaseContent.WhiteTex);
			nextClmn.x += nextClmn.width + gapHorizontal * 2 + lineThickness; nextClmn.width = clmnSizes[1];
			QOLCityProjectDef_Crisis crisis = project as QOLCityProjectDef_Crisis;
			if ( crisis != null ) {
				Rect barRect = new Rect(nextClmn);
				GUI.DrawTexture(barRect, BaseContent.BlackTex);
				barRect = barRect.ContractedBy(3f);
				
				GUI.DrawTexture(barRect, Materials.DefaultBarBgTex);
				
				QOLCityProject_Crisis instanceWithMaxProgress = null;
				foreach ( QOLCityProject proj in projectInstances ) {
					QOLCityProject_Crisis crisisState = proj as QOLCityProject_Crisis;
					if ( crisisState != null ) {
						if ( instanceWithMaxProgress == null || crisisState.progress > instanceWithMaxProgress.progress ) {
							instanceWithMaxProgress = crisisState;
						}
					}
				}
				
				if ( instanceWithMaxProgress != null ) {
					float max = crisis.maxDisplayProgression, min = crisis.minDisplayProgression;
					float spread = Math.Abs(max - min);
					barRect.width = ((Mathf.Clamp(instanceWithMaxProgress.progress,min,max) - min)/spread) * barRect.width;
					GUI.DrawTexture(barRect, Materials.BarFullTexHor);
					if ( Mouse.IsOver(nextClmn) ) {
						TooltipHandler.TipRegion(nextClmn, ()=>min.ToString("0")+">"+instanceWithMaxProgress.progress.ToString("0")+"<"+max.ToString("0"), 1);
					}
				} else {
					if ( Mouse.IsOver(nextClmn) ) {
						TooltipHandler.TipRegion(nextClmn, ()=>"InactiveFacility".Translate(), 1);
					}
				}
			}
			
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityAchievementRenderer : BaseRenderer {
		protected const float gapHorizontal = 4f;
		protected const float lineThickness = 1f;
		protected const float iconSize = 16f;
		protected const GameFont font = GameFont.Small;
		static protected float height = -1;
		protected Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup;
		public QOLCityAchievementRenderer(Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup) {
			this.projectLookup = projectLookup;
		}
		override public float getHeight() {
			if ( height < 0f ) {
				height = Mathf.Max(QOLMod.LineHeight(font),iconSize);
			}
			return height;
		}
		override public float getWidth() {
			return 150f;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			GUI.color = Color.white;
			
			Pair<QOLCityProjectDef,List<QOLCityProject>> p = projectLookup(flow);
			QOLCityProjectDef project = p.First;
			List<QOLCityProject> projectInstances = p.Second;
			
			if ( projectInstances.NullOrEmpty() ) {
				inRect.height = 0;
				return;
			}
			
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			float lineHeight = QOLMod.LineHeight(font), height = getHeight();
			Rect nextClmnText = new Rect(inRect.x + gapHorizontal, inRect.y+(getHeight()-lineHeight)/2f, 150f, lineHeight);
			Rect nextClmn = new Rect(inRect.x + gapHorizontal, inRect.y, 150f, height);
			GUI.Label(nextClmnText, project.label, Text.CurFontStyle);
			
			//Tooltip
			if ( Mouse.IsOver(nextClmnText) ) {
				TooltipHandler.TipRegion(nextClmnText, project.Tooltip, 1);
			}
			
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityBuffRenderer : QOLCityAchievementRenderer {
		public QOLCityBuffRenderer(Func<Flow,Pair<QOLCityProjectDef,List<QOLCityProject>>> projectLookup) : base(projectLookup) {}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			Pair<QOLCityProjectDef,List<QOLCityProject>> p = projectLookup(flow);
			QOLCityProjectDef project = p.First;
			List<QOLCityProject> projectInstances = p.Second;
			
			if ( projectInstances.NullOrEmpty() ) {
				inRect.height = 0;
				return;
			}
			
			int activeInstances = projectInstances.Count(pi=>pi.IsActive);
			
			GUI.color = activeInstances > 0 ? Color.white : Color.gray;
			
			Text.Font = font;
			Text.Anchor = TextAnchor.MiddleLeft;
			float lineHeight = QOLMod.LineHeight(font), height = getHeight();
			Rect nextClmnText = new Rect(inRect.x + gapHorizontal, inRect.y+(getHeight()-lineHeight)/2f, 150f, lineHeight);
			Rect nextClmn = new Rect(inRect.x + gapHorizontal, inRect.y, 150f, height);
			GUI.Label(nextClmnText, project.label+" x "+activeInstances.ToString("0"), Text.CurFontStyle);
			
			GUI.color = Color.white;
			
			//Tooltip
			if ( Mouse.IsOver(nextClmnText) ) {
				TooltipHandler.TipRegion(nextClmnText, project.Tooltip, 1);
			}
			
			GUI.color = oldColor;
			Text.Font = oldFont;
			Text.Anchor = oldAnchor;
		}
	}
	
	public class QOLCityStatsRenderer : ParentRendererWithIteration {
		protected float lastHeight;
		public QOLCityStatsRenderer() {
			lastHeight = 0;
			AddChild(new QOLCityResourceRenderer(f=>(KeyValuePair<QOLCityResourceDef,NumericAdjustment>)((IterationItem)f.pageScope["curItem"]).curItem));
			AddChild(new QOLCityStatsNeedRenderer(f=>(KeyValuePair<QOLCityNeedDef,ProvidedAndRequired>)((IterationItem)f.pageScope["curItem"]).curItem));
			AddChild(new QOLCityStatsIndicatorRenderer(f=>(KeyValuePair<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>>)((IterationItem)f.pageScope["curItem"]).curItem));
		}
		override public float getHeight() {
			return lastHeight;
		}
		override public float getWidth() {
			return 680f;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			QOLCity city = (QOLCity)flow.flowScope["city"];
			float usedHeight = 0;
			beginIteration(flow);
			try {
				BaseRenderer renderer = childs[0];
				float width = renderer.getWidth(), gap = 4f, maxWidth = getWidth();
				Rect nextRect = new Rect(inRect.x,inRect.y + usedHeight,width,renderer.getHeight());
				foreach ( KeyValuePair<QOLCityResourceDef,NumericAdjustment> resource in city.stateProjectedState.resources ) {
					if ( nextRect.xMax > maxWidth ) {
						nextRect.x = inRect.x;
						nextRect.y += gap + nextRect.height;
					}
					advanceIteration(resource);
					renderer.DoComponentContents(nextRect, flow);
					nextRect.x += width + gap;
				}
				usedHeight = nextRect.yMax - inRect.yMin;
			} finally {
				endIteration(flow);
			}
			
			//seperator line
			GUI.DrawTexture(new Rect(inRect.x + 16f, inRect.y + usedHeight + 4f, inRect.width - 32f, 2f),BaseContent.WhiteTex);
			usedHeight += 4f * 2f + 2f;
			
			beginIteration(flow);
			try {
				BaseRenderer renderer = childs[1];
				float width = renderer.getWidth(), gap = 4f, maxWidth = getWidth();
				Rect nextRect = new Rect(inRect.x,inRect.y + usedHeight,width,renderer.getHeight());
				foreach ( KeyValuePair<QOLCityNeedDef,ProvidedAndRequired> need in city.stateProjectedState.needs ) {
					if ( nextRect.xMax > maxWidth ) {
						nextRect.x = inRect.x;
						nextRect.y += gap + nextRect.height;
					}
					advanceIteration(need);
					renderer.DoComponentContents(nextRect, flow);
					nextRect.x += width + gap;
				}
				usedHeight = nextRect.yMax - inRect.yMin;
			} finally {
				endIteration(flow);
			}
			
			//seperator line
			GUI.DrawTexture(new Rect(inRect.x + 16f, inRect.y + usedHeight + 4f, inRect.width - 32f, 2f),BaseContent.WhiteTex);
			usedHeight += 4f * 2f + 2f;
			
			beginIteration(flow);
			try {
				BaseRenderer renderer = childs[2];
				float width = renderer.getWidth(), gap = 4f, maxWidth = getWidth();
				Rect nextRect = new Rect(inRect.x,inRect.y + usedHeight,width,renderer.getHeight());
				foreach ( KeyValuePair<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>> indicator in city.stateProjectedState.activeStages ) {
					if ( nextRect.xMax > maxWidth ) {
						nextRect.x = inRect.x;
						nextRect.y += gap + nextRect.height;
					}
					advanceIteration(indicator);
					renderer.DoComponentContents(nextRect, flow);
					nextRect.x += width + gap;
				}
				usedHeight = nextRect.yMax - inRect.yMin;
			} finally {
				endIteration(flow);
			}
			
			lastHeight = usedHeight;
		}
	}
	
	public class QOLCityResourceRenderer : BaseRenderer {
		protected const float gapBetweenIconAndLabel = 4f;
		protected const float iconSize = 32f;
		protected const GameFont font = GameFont.Medium;
		static protected float width = -1;
		static protected float height = -1;
		protected Func<Flow,KeyValuePair<QOLCityResourceDef,NumericAdjustment>> resourceLookup;
		public QOLCityResourceRenderer(Func<Flow,KeyValuePair<QOLCityResourceDef,NumericAdjustment>> resourceLookup) {
			this.resourceLookup = resourceLookup;
		}
		override public float getHeight() {
			if ( height < 0f ) {
				height = Mathf.Max(QOLMod.LineHeight(font),iconSize);
			}
			return height;
		}
		override public float getWidth() {
			if ( width < 0f ) {
				width = QOLMod.CalcSize("-00,0", GameFont.Medium, TextAnchor.UpperLeft, false).x + gapBetweenIconAndLabel + iconSize; 
			}
			return width;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			KeyValuePair<QOLCityResourceDef,NumericAdjustment> res = resourceLookup(flow);
			if ( ButtonImage(inRect, res.Key, (res.Value.Value).ToString("00.0")) ) {
				QOLCity city = (QOLCity)flow.flowScope["city"];
				Pawn negotiator = (Pawn)flow.flowScope["negotiator"];
				res.Key.callback.Callback(res.Key,city,negotiator);
			}
		}
		
		public static bool ButtonImage(Rect butRect, QOLCityResourceDef res, string label) {
			if (Mouse.IsOver(butRect)) {
				GUI.color = GenUI.MouseoverColor;
			} else {
				GUI.color = Color.white;
			}
			Rect rect = new Rect(butRect.x, butRect.y+(iconSize-butRect.height)/2f, iconSize, iconSize);
			GUI.DrawTexture(rect, res.Icon);
			float lineHeight = QOLMod.LineHeight(font);
			rect.x += rect.width + gapBetweenIconAndLabel;
			rect.y = butRect.y+(lineHeight-butRect.height)/2f;
			rect.width = butRect.width-rect.x+butRect.x;
			rect.height = lineHeight;

			bool wasCapped;
			string labelCapped = QOLMod.CapString(label,rect.width,"...",out wasCapped,font);
			GUI.color = Color.white;
			GUI.Label(rect, labelCapped, Text.CurFontStyle);
			TooltipHandler.TipRegion(butRect, ()=>res.label+"("+label+")"+"\n\n"+res.description, 1);
			return Widgets.ButtonInvisible(butRect, false);
		}
	}
	
	public class QOLCityStatsNeedRenderer : BaseRenderer {
		protected Func<Flow,KeyValuePair<QOLCityNeedDef,ProvidedAndRequired>> needLookup;
		protected const float barHeight = 22f;
		protected const float barWidth = 228f;//264 width
		protected const float barHeightHalf = barHeight / 2f;
		protected const float tendencyIndicatorMargin = 2f;
		protected const float markerMargin = 0f;
		protected const float topLabelMargin = 2f;
		protected const float markerSize = 8f;
		protected const float tendencyIndicatorHeight = 16f;
		protected const float tendencyIndicatorWidth = tendencyIndicatorHeight / 2f;
		static readonly Color topLabelColor = Color.white;
		static readonly protected Color tendencyIndicatorLeftColor =  Color.red;
		static readonly protected Color tendencyIndicatorRightColor =  Color.green;
		static readonly Color markerColor =  Color.white;
		protected const GameFont font = GameFont.Medium;
		protected TranslationRequest trLabelProvided = new TranslationRequest("Provided"), trLabelRequired = new TranslationRequest("Required");
		
		public QOLCityStatsNeedRenderer(Func<Flow,KeyValuePair<QOLCityNeedDef,ProvidedAndRequired>> needLookup) {
			this.needLookup = needLookup;
		}
		override public float getHeight() {
			return QOLMod.LineHeight(font);// + barHeight + markerSize + markerMargin + topLabelMargin;
		}
		override public float getWidth() {
			return 164f;//tendencyIndicatorMargin * 2 + tendencyIndicatorHeight * 2 + barWidth;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			QOLCity city = (QOLCity)flow.flowScope["city"];
			KeyValuePair<QOLCityNeedDef,ProvidedAndRequired> kvpSocietyIndicator = needLookup(flow);
			QOLCityNeedDef need = kvpSocietyIndicator.Key;
			ProvidedAndRequired par = kvpSocietyIndicator.Value;
			float needValue = par.provided.Value;
			float requirementValue = par.required.Value;
			
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			Text.Font = font;
			Text.Anchor = TextAnchor.UpperLeft;
			GUI.color = needValue >= requirementValue ? Color.white : Color.red;
			Rect labelRect = new Rect(inRect.x + 4f, inRect.y+(inRect.height-Text.LineHeight)/2f, 100f, Text.LineHeight);
			bool wasCapped;
			string labelToRender = need.label;
			string labelCapped = QOLMod.CapString(labelToRender,labelRect.width,"...",out wasCapped,Text.Font,Text.Anchor);
			displayToolTipIfNecessary(labelRect, flow, wasCapped ? labelToRender : null);
			Widgets.Label(labelRect, labelCapped);
			
			labelRect.x += labelRect.width + 4f;
			labelRect.width = 50f;
			labelToRender = (needValue-requirementValue).ToString("###0.##");
			labelCapped = QOLMod.CapString(labelToRender,labelRect.width,"...",out wasCapped,Text.Font,Text.Anchor);
			
			tooltipExpr = f => new StringBuilder().Append(needValue.ToString("###0.## - ")).Append(requirementValue.ToString("###0.## = ")).AppendLine(labelToRender)
				.AppendLine(par.provided.ToLabeledString(TranslationCache.getFormattedTranslation(trLabelProvided)))
				.AppendLine(par.required.ToLabeledString(TranslationCache.getFormattedTranslation(trLabelRequired))).ToString();
			displayToolTipIfNecessary(labelRect, flow, null);
			tooltipExpr = null;
			Widgets.Label(labelRect, labelCapped);
			
			Text.Anchor = oldAnchor;
			GUI.color = oldColor;
			Text.Font = oldFont;
		}
	}
	
	public class QOLCityStatsIndicatorRenderer : BaseRenderer {
		protected Func<Flow,KeyValuePair<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>>> societyIndicatorLookup;
		protected const float barHeight = 22f;
		protected const float barWidth = 228f;//264 width
		protected const float barHeightHalf = barHeight / 2f;
		protected const float tendencyIndicatorMargin = 2f;
		protected const float markerMargin = 0f;
		protected const float topLabelMargin = 2f;
		protected const float bottomLabelMargin = -2f;
		protected const float markerSize = 8f;
		protected const float tendencyIndicatorHeight = 16f;
		protected const float tendencyIndicatorWidth = tendencyIndicatorHeight / 2f;
		static readonly Color topLabelColor = Color.white;
		static readonly protected Color bottomLabelColor = new Color(0.2f, 0.8f, 0.85f);
		static readonly protected Color tendencyIndicatorColor =  new Color(126f/255f,199f/255f,88f/255f);
		static readonly Color markerColor =  Color.white;
		protected const GameFont font = GameFont.Medium;
		
		public QOLCityStatsIndicatorRenderer(Func<Flow,KeyValuePair<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>>> societyIndicatorLookup) {
			this.societyIndicatorLookup = societyIndicatorLookup;
		}
		override public float getHeight() {
			return QOLMod.LineHeight(font) * 2 + barHeight + markerSize + markerMargin + topLabelMargin;
		}
		override public float getWidth() {
			return tendencyIndicatorMargin * 2 + tendencyIndicatorHeight * 2 + barWidth;
		}
		override public void DoComponentContents(Rect inRect, Flow flow) {
			KeyValuePair<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>> kvpSocietyIndicator = societyIndicatorLookup(flow);
			QOLSocietyIndicatorDef societyIndicator = kvpSocietyIndicator.Key;
			QOLSocietyIndicatorStage activeStage = kvpSocietyIndicator.Value.First;
			float societyIndicatorValue = kvpSocietyIndicator.Value.Second, lastSocietyIndicatorValue = societyIndicatorValue;
			
			QOLCity city = (QOLCity)flow.flowScope["city"];
			if ( city.stateLastSimulatedTick != null ) {
				Pair<QOLSocietyIndicatorStage,float> lastValues;
				if ( city.stateLastSimulatedTick.activeStages.TryGetValue(societyIndicator, out lastValues) ) {
					if ( lastValues.First == activeStage ) 
						lastSocietyIndicatorValue = lastValues.Second;
				}
			}
			
			Color oldColor = GUI.color;
			GameFont oldFont = Text.Font;
			TextAnchor oldAnchor = Text.Anchor;
			
			//labels
			Text.Font = font;
			float lineHeight = QOLMod.LineHeight(Text.Font);
			Text.Anchor = TextAnchor.UpperLeft;
			GUI.color = topLabelColor;
			GUI.Label(new Rect(inRect.x + tendencyIndicatorWidth + tendencyIndicatorMargin, inRect.y, barWidth, lineHeight),
			          societyIndicator.label, Text.CurFontStyle);
			GUI.color = bottomLabelColor;
			
			if ( activeStage != null ) {
				GUI.Label(new Rect(inRect.x + tendencyIndicatorWidth + tendencyIndicatorMargin, inRect.y + lineHeight + barHeight + markerSize + topLabelMargin + bottomLabelMargin, barWidth, lineHeight),
				          activeStage.label, Text.CurFontStyle);
			}
			
			//indicator left and right
			Rect barRect = new Rect(inRect.x + tendencyIndicatorWidth + tendencyIndicatorMargin,inRect.y + lineHeight + topLabelMargin,barWidth,barHeight);
			if ( Mouse.IsOver(barRect) ) {
				if (Event.current.type == EventType.MouseDown && Event.current.button == 0) {
					Event.current.Use();
					GUIUtility.keyboardControl = 0;
					Find.WindowStack.Add(new QOLCityDialog_PawnIndicatorStages(societyIndicator,city.stateProjectedState));
				} else {
					TooltipHandler.TipRegion(barRect, ()=>societyIndicator.Tooltip(city.stateProjectedState.stageValues[societyIndicator], activeStage), 1);
				}
			}
			
			if ( societyIndicatorValue > lastSocietyIndicatorValue ) {
				Rect position = new Rect(barRect.xMax + tendencyIndicatorMargin, barRect.y + (barRect.height-tendencyIndicatorHeight)/2f, tendencyIndicatorWidth, tendencyIndicatorHeight);
				GUI.color = tendencyIndicatorColor;
				GUI.DrawTexture(position, Materials.FillArrowTexRight);
			} else if ( societyIndicatorValue < lastSocietyIndicatorValue ) {
				Rect position = new Rect(barRect.xMin - tendencyIndicatorWidth - tendencyIndicatorMargin, barRect.y + (barRect.height-tendencyIndicatorHeight)/2f, tendencyIndicatorWidth, tendencyIndicatorHeight);
				GUI.color = tendencyIndicatorColor;
				GUI.DrawTexture(position, Materials.FillArrowTexLeft);
			}
			
			Text.Anchor = oldAnchor;
			GUI.color = oldColor;
			Text.Font = oldFont;
			
			GUI.DrawTexture(barRect, BaseContent.BlackTex);
			barRect = barRect.ContractedBy(3f);
			
			GUI.DrawTexture(barRect, Materials.DefaultBarBgTex);
			
			//marker
			GUI.color = markerColor;
			GUI.DrawTexture(new Rect(barRect.x + barRect.width * societyIndicatorValue - markerSize / 2f, barRect.yMax + 3f, 
			                         markerSize, markerSize), Materials.BarInstantMarkerTex);

			barRect.width = barRect.width * societyIndicatorValue;
			GUI.DrawTexture(barRect, Materials.BarFullTexHor);
		}
	}
}
