﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 20.01.2019
 * Time: 23:09
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using RIMMSqol.remnantcolony.actions;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.listeners
{
	/// <summary>
	/// Description of QOLCityListener_ActionCraft.
	/// </summary>
	public class QOLCityListener_ActionCraft : QOLCityListener
	{
		protected int workEffectively = 0;
		
		public QOLCityListener_ActionCraft(QOLCityListenerDef def,IQOLCityListenerOwner owner) : base(def,owner) {}
		
		public override void OnEvent(QOLCity city, QOLCityEvent e) {
			QOLCityAction_Craft action = owner as QOLCityAction_Craft;
			if ( action != null ) {
				QOLCityActionConfiguration_Craft conf = action.def.action as QOLCityActionConfiguration_Craft;
				if ( conf != null ) {
					List<CraftOrder> orders = action.orders;
					if ( e.EventId != QOLCityEvents.ConsumeContributions ) {
						QOLCityEvent_State stateEvent = e as QOLCityEvent_State;
						if ( stateEvent != null ) {
							if ( action.WorkAllocated > 0 ) {
								float workMaximal = city.LabourBalance(stateEvent.State);
								workMaximal = Math.Max(0,workMaximal);
								if ( workMaximal < action.WorkAllocated ) {
									workEffectively = Mathf.FloorToInt(workMaximal);
								} else {
									workEffectively = action.WorkAllocated;
								}
								workEffectively = Math.Min(workEffectively,city.projects.Count(p=>p.IsActive && p.def.actions.Contains(action.def)) * conf.maxLabourPerInstance);
								//if not all crafting is needed we can reduce the labour drain for next turn
								if ( workEffectively > 0 && conf != null ) {
									if ( !orders.NullOrEmpty() ) {
										int workLeft = 0;
										foreach ( CraftOrder order in orders ) {
											workLeft += order.workLeft;
											if ( workLeft > workEffectively ) break;
										}
										workEffectively = Math.Min(workEffectively, workLeft);
									} else workEffectively = 0;
								}
								ProvidedAndRequired par = city.LabourPaR(stateEvent.State);
								par.required.flat += workEffectively;
							}
						} else {
							Log.Error("Listener \""+typeof(QOLCityListener_ActionRepair).Name+"\" was called with unexpected event type. Either \""+QOLCityEvents.ConsumeContributions+"\" or a state event are expected!");
						}
					} else {
						if ( !orders.NullOrEmpty() ) {
							int workToAssign = Mathf.RoundToInt(workEffectively*conf.workMultiplier);
							
							CraftOrder nextOrder = orders.FirstOrDefault();
							while ( nextOrder != null ) {
								nextOrder.workLeft -= workToAssign;
								if ( nextOrder.workLeft <= 0 ) {
									workToAssign = Math.Abs(nextOrder.workLeft);
									nextOrder.workLeft = 0;
									nextOrder.giveProductsTo(city);
									
									Find.LetterStack.ReceiveLetter("OrderCompleted".Translate(nextOrder.recipe.label), 
								                               "OrderCompletedNotification".Translate(city.GetCallLabel(), nextOrder.recipe.label),
								                               LetterDefOf.PositiveEvent, city, null, null);
									orders.Remove(nextOrder);
									nextOrder = orders.FirstOrDefault();
								} else {
									break;
								}
							}
						}
						workEffectively = 0;
					}
				}
			} else {
				Log.Error("Listener \""+typeof(QOLCityListener_ActionCraft).Name+"\" was registered on an incompatible owner \""+owner+"\"!");
			}
		}
	}
}
