﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 20.01.2019
 * Time: 22:05
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Linq;
using System.Collections.Generic;
using System.Reflection;
using RIMMSqol.genericSettings;
using RIMMSqol.remnantcolony.actions;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.listeners
{
	/// <summary>
	/// Description of QOLCityListener_ActionRepair.
	/// </summary>
	public class QOLCityListener_ActionRepair : QOLCityListener
	{
		protected int workEffectively = 0;
		protected MethodInfo cpwTryGetComp, cpwRemoveLootedWeaponFlag;
		protected PropertyInfo cpwIsUsedWeapon;
		
		public QOLCityListener_ActionRepair(QOLCityListenerDef def,IQOLCityListenerOwner owner) : base(def,owner) {
			Type compProfitableWeapons = DefSettingsField.GetTypeFromAnyAssemblyVersion("ProfitableWeapons.CompLootedWeapon, ProfitableWeapons, Version=1.2.6.0, Culture=neutral, PublicKeyToken=null");
			if ( compProfitableWeapons != null ) {
				cpwTryGetComp = typeof(ThingCompUtility).GetMethod("TryGetComp").MakeGenericMethod(compProfitableWeapons);
				cpwIsUsedWeapon = compProfitableWeapons.GetProperty("IsUsedWeapon");
				cpwRemoveLootedWeaponFlag = compProfitableWeapons.GetMethod("RemoveLootedWeaponFlag");
			} else {
				cpwTryGetComp = null;
				cpwIsUsedWeapon = null;
				cpwRemoveLootedWeaponFlag = null;
			}
		}
		
		public override void OnEvent(QOLCity city, QOLCityEvent e) {
			QOLCityAction_Repair action = owner as QOLCityAction_Repair;
			if ( action != null ) {
				if ( e.EventId != QOLCityEvents.ConsumeContributions ) {
					QOLCityEvent_State stateEvent = e as QOLCityEvent_State;
					if ( stateEvent != null ) {
						if ( action.WorkAllocated > 0 ) {
							float workMaximal = city.LabourBalance(stateEvent.State);
							workMaximal = Math.Max(0,workMaximal);
							if ( workMaximal < action.WorkAllocated ) {
								workEffectively = Mathf.FloorToInt(workMaximal);
							} else {
								workEffectively = action.WorkAllocated;
							}
							//if not all repairs are needed we can reduce the labour drain for next turn
							QOLCityActionConfiguration_Repair conf = action.def.action as QOLCityActionConfiguration_Repair;
							if ( conf != null ) {
								workEffectively = Math.Min(workEffectively,city.projects.Count(p=>p.IsActive && p.def.actions.Contains(action.def)) * conf.MaxCyclePerInstance);
								if ( workEffectively > 0 ) {
									List<ThingCategoryDef> tc = conf.AllCategories;
									int missingHealthInTotal = city.GetDirectlyHeldThings().Where(thing=>thing.HitPoints < thing.MaxHitPoints && thing.def.thingCategories != null && tc.SharesElementWith(thing.def.thingCategories)).Sum(thing=>thing.MaxHitPoints-thing.HitPoints);
									missingHealthInTotal = Mathf.CeilToInt(missingHealthInTotal / conf.hitPointMultiplier);
									if ( missingHealthInTotal < workEffectively ) {
										workEffectively = missingHealthInTotal;
									}
								}
							}
							ProvidedAndRequired par = city.LabourPaR(stateEvent.State);
							par.required.flat += workEffectively;
						}
					} else {
						Log.Error("Listener \""+typeof(QOLCityListener_ActionRepair).Name+"\" was called with unexpected event type. Either \""+QOLCityEvents.ConsumeContributions+"\" or a state event are expected!");
					}
				} else {
					QOLCityActionConfiguration_Repair conf = action.def.action as QOLCityActionConfiguration_Repair;
					if ( conf != null ) {
						if ( !conf.categories.NullOrEmpty() ) {
							List<ThingCategoryDef> tc = conf.AllCategories;
							if ( workEffectively > 0 ) {
								int hitPointsToRepair = Mathf.RoundToInt(workEffectively * conf.hitPointMultiplier);
								while ( hitPointsToRepair > 0 ) {
									Thing t;
									if ( city.GetDirectlyHeldThings().Where(thing=>
									                                        thing.HitPoints < thing.MaxHitPoints && thing.def.thingCategories != null &&
									                                        tc.SharesElementWith(thing.def.thingCategories)).TryRandomElement(out t) ) {
										int missingHitpoints = t.MaxHitPoints - t.HitPoints;
										if ( hitPointsToRepair <= missingHitpoints ) {
											t.HitPoints += hitPointsToRepair;
										} else {
											t.HitPoints = t.MaxHitPoints;
										}
										hitPointsToRepair -= missingHitpoints;
									} else {
										break;
									}
								}
							}
							workEffectively = 0;
							//Clearing the dead mans apparel flag on all things that are fully repaired
							foreach ( Thing t in city.GetDirectlyHeldThings().Where(thing=>thing.HitPoints >= thing.MaxHitPoints && IsDeadmansApparel(thing) &&
							                                                        thing.def.thingCategories != null && tc.SharesElementWith(thing.def.thingCategories)) ) {
								if ( t.def.IsApparel ) {
									try {
										typeof(Apparel).GetField("wornByCorpseInt", BindingFlags.Instance|BindingFlags.NonPublic).SetValue(t,false);
									} catch {
										Log.Error("Failed to clear dead man status of apparel \""+t.def.defName+"\".");
									}
								}
								if ( t.def.IsWeapon && cpwTryGetComp != null ) {
									try {
										ThingComp comp = cpwTryGetComp.Invoke(null,new object[]{t}) as ThingComp;
										if ( comp != null ) {
											cpwRemoveLootedWeaponFlag.Invoke(comp,null);
										}
									} catch {
										Log.Error("Failed to clear looted status of weapon \""+t.def.defName+"\".");
									}
								}
							}
						} else {
							Log.Error("No thingCategories defined in action segment of def "+def.defName+"!");
						}
					}
				}
			} else {
				Log.Error("Listener \""+typeof(QOLCityListener_ActionRepair).Name+"\" was registered on an incompatible owner \""+owner+"\"!");
			}
		}
		
		protected bool IsDeadmansApparel(Thing t) {
			if ( t.def.IsApparel && t.def.apparel.careIfWornByCorpse ) {
				Apparel apparel = t as Apparel;
				if ( apparel != null && apparel.WornByCorpse ) return true;
			}
			
			if ( t.def.IsWeapon && cpwTryGetComp != null ) {
				ThingComp comp = cpwTryGetComp.Invoke(null,new object[]{t}) as ThingComp;
				if ( comp != null ) {
					return (bool)cpwIsUsedWeapon.GetValue(comp,null);
				}
			}
			
			return false;
		}
	}
}
