﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 22.01.2019
 * Time: 23:01
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using RIMMSqol.remnantcolony.projects;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.listeners
{
	/// <summary>
	/// Description of QOLCityListener_CrisisProgression.
	/// </summary>
	public class QOLCityListener_CrisisProgression : QOLCityListener
	{
		protected QOLCityProject_Crisis proj;
		protected QOLCityProjectDef_Crisis conf;
		
		public QOLCityListener_CrisisProgression(QOLCityListenerDef def,IQOLCityListenerOwner owner) : base(def,owner) {
			proj = owner as QOLCityProject_Crisis;
			if ( proj == null ) {
				Log.Error("Listener definition "+def.defName+" is paired with illegal owner type, the owner must implement "+typeof(QOLCityProject_Crisis).Name+"!");
				return;
			}
			conf = proj.def as QOLCityProjectDef_Crisis;
			if ( conf == null ) {
				Log.Error("Listener definition "+def.defName+" is paired with illegal owner configuration type, the owners definition must implement "+typeof(QOLCityProjectDef_Crisis).Name+"!");
				return;
			}
			proj.progress = Mathf.Clamp(proj.progress, conf.minProgression, conf.maxProgression);
		}
		
		public override void OnEvent(QOLCity city, QOLCityEvent e) {
			if ( conf == null ) return;
			
			proj.progress -= conf.dailyDecrease;
			
			if ( !conf.progression.NullOrEmpty() ) {
				foreach ( QOLCityProjectDef_Crisis_RequirementProgress rp in conf.progression ) {
					if ( rp.requirements.NullOrEmpty() || rp.requirements.TrueForAll(r=>r.IsValid(city)) ) {
						proj.progress += rp.dailyProgress;
					}
				}
			}
			
			proj.progress = Mathf.Clamp(proj.progress, conf.minProgression, conf.maxProgression);
			
			if ( !conf.triggers.NullOrEmpty() ) {
				foreach ( QOLCityProjectDef_Crisis_RequirementTrigger rt in conf.triggers ) {
					if ( rt.requirements.NullOrEmpty() || rt.requirements.TrueForAll(r=>r.IsValid(city)) ) {
						if ( rt.trigger != null ) rt.trigger.Trigger(this, city);
					}
				}
			}
		}
	}
}
