﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.02.2019
 * Time: 06:51
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Linq;
using System.Collections.Generic;
using RIMMSqol.translations;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.triggers
{
	/// <summary>
	/// Description of QOLCityTrigger_IncidentFamilyReunification.
	/// </summary>
	public class QOLCityTrigger_IncidentFamilyReunification : QOLCityTriggerImpl {
		[Unsaved]
		protected TranslationRequest translationRequestDescription = null;
		
		override public void Trigger(object owner, QOLCity city) {
			//TODO add configurable parameter for raid
			Map playerSettlement = Find.AnyPlayerHomeMap;
			if ( playerSettlement == null ) {
				return;
			}
			IntVec3 spawnSpot;
			if ( !CellFinder.TryFindRandomEdgeCellWith(c => playerSettlement.reachability.CanReachColony(c) && !c.Fogged(playerSettlement), 
			                                           playerSettlement, CellFinder.EdgeRoadChance_Neutral, out spawnSpot) ) {
				return;
			}
			List<Pawn> pawnsWithFamilyInForeignFaction = new List<Pawn>();
			foreach ( KeyValuePair<Pawn,QOLCitySocialStandingDef> kvp in city.population.citizens ) {
				Pawn p = kvp.Key;
				foreach ( DirectPawnRelation rel in p.relations.DirectRelations ) {
					if ( (rel.otherPawn.Faction == null || !rel.otherPawn.Faction.IsPlayer) && !p.Dead && !p.Spawned ) {
						pawnsWithFamilyInForeignFaction.Add(p);
						break;
					}
				}
			}
			if ( !pawnsWithFamilyInForeignFaction.NullOrEmpty() ) {
				/*
				 * Pick a random pawn that qualifies and spawn their family.
				 */
				Pawn p = pawnsWithFamilyInForeignFaction.RandomElement();
				List<DirectPawnRelation> familyInForeignFaction = new List<DirectPawnRelation>();
				foreach ( DirectPawnRelation rel in p.relations.DirectRelations ) {
					if ( (rel.otherPawn.Faction == null || !rel.otherPawn.Faction.IsPlayer) && !p.Dead && !p.Spawned ) {
						familyInForeignFaction.Add(rel);
					}
				}
				Faction foreignHostileFaction = null;
				foreach ( DirectPawnRelation familyMemberRelation in familyInForeignFaction ) {
					Pawn familyMember = familyMemberRelation.otherPawn;
					if ( foreignHostileFaction == null && familyMember.Faction.HostileTo(Faction.OfPlayer) ) {
						foreignHostileFaction = familyMember.Faction;
					}
					GenSpawn.Spawn(familyMember, spawnSpot, playerSettlement, WipeMode.Vanish);
					familyMember.SetFaction(Faction.OfPlayer, null);
					if ( !CellFinder.TryFindRandomSpawnCellForPawnNear(spawnSpot, playerSettlement, out spawnSpot) ) {
						break;
					}
				}
				CameraJumper.TryJump(familyInForeignFaction[0].otherPawn);
				
				/*
				 * Queue the raid from a hostile faction that at least one family member had.
				 */
				if ( foreignHostileFaction != null ) {
					int @int = Rand.Int;
					IncidentParms raidParms = StorytellerUtility.DefaultParmsNow(IncidentCategoryDefOf.ThreatBig, playerSettlement);
					raidParms.forced = true;
					raidParms.faction = foreignHostileFaction;
					raidParms.raidStrategy = RaidStrategyDefOf.ImmediateAttack;
					raidParms.raidArrivalMode = PawnsArrivalModeDefOf.EdgeWalkIn;
					raidParms.spawnCenter = spawnSpot;
					raidParms.points = Mathf.Max(raidParms.points * new FloatRange(1f, 1.6f).RandomInRange, foreignHostileFaction.def.MinPointsToGeneratePawnGroup(PawnGroupKindDefOf.Combat));
					raidParms.pawnGroupMakerSeed = new int?(@int);
					
					Find.Storyteller.incidentQueue.Add(new QueuedIncident(new FiringIncident(IncidentDefOf.RaidEnemy, null, raidParms), Find.TickManager.TicksGame + 4000, 0));
				}
				
				/*
				 * Notify player about result.
				 */
				if ( foreignHostileFaction != null ) {
					Find.LetterStack.ReceiveLetter("FamilyReunificationTitle".Translate(),
						"FamilyReunificationFollowed".Translate(new NamedArgument(p,"colonist"),
					                                                                       new NamedArgument(familyInForeignFaction.Select(rel=>rel.otherPawn.Name.ToStringFull+"("+rel.def.label+")").ToCommaList(true),"familyMembers"),
                               new NamedArgument(foreignHostileFaction,"faction")).AdjustedFor(p, "colonist"),
						LetterDefOf.ThreatBig, familyInForeignFaction[0].otherPawn, foreignHostileFaction, null);
				} else {
					Find.LetterStack.ReceiveLetter("FamilyReunificationTitle".Translate(),
						"FamilyReunification".Translate(new NamedArgument(p,"colonist"),
                               new NamedArgument(familyInForeignFaction.Select(rel=>rel.otherPawn.Name.ToStringFull+"("+rel.def.label+")").ToCommaList(true),"familyMembers")).AdjustedFor(p, "colonist"),
						LetterDefOf.PositiveEvent, familyInForeignFaction[0].otherPawn, foreignHostileFaction, null);
				}
				
				//if this trigger was called via a listener we want to mark the responsible event
				QOLCityEvent e = owner as QOLCityEvent;
				if ( e != null ) e.StopProcessing();
			}
		}
		
		override protected TranslationRequest TranslationRequestDescription() {
			if ( translationRequestDescription == null ) {
				translationRequestDescription = new TranslationRequest(typeof(QOLCityTrigger_IncidentFamilyReunification).Name);
			}
			return translationRequestDescription;
		}
		
	}
}
