﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 05.12.2017
 * Time: 14:26
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using UnityEngine;
using Verse;

namespace RIMMSqol.renderers
{
	/// <summary>
	/// Description of ButtonRenderer.
	/// </summary>
	public class ButtonRenderer : BaseRenderer {
		protected Texture2D iconTexture;
        protected Func<Flow, Texture2D> exprIcon;
        protected Action<Flow> onClick;
		protected float height = -1, width = -1;
		protected Func<Flow,bool> hiddenOn;
        public bool DrawBackground { get; set; } = false;
		public override float getHeight() {
			if ( height >= 0 ) return height;
			return iconTexture.height;
		}
		public override float getWidth() {
			if ( width >= 0 ) return width;
			return iconTexture.width;
		}
		public ButtonRenderer hidden(Func<Flow,bool> hiddenOn) {
			this.hiddenOn = hiddenOn;
			return this;
		}
		public ButtonRenderer(Texture2D iconTexture, Action<Flow> onClick, float width = -1, float height = -1) {
			this.iconTexture = iconTexture;
			this.onClick = onClick;
			this.width = width;
			this.height = height;
		}
        public ButtonRenderer(Func<Flow, Texture2D> exprIcon, Action<Flow> onClick, float width, float height)
        {
            this.exprIcon = exprIcon;
            this.onClick = onClick;
            this.width = width;
            this.height = height;
        }
        public override void DoComponentContents(Rect inRect, Flow flow) {
			if ( hiddenOn != null && hiddenOn(flow) ) {
				return;
			}
			Rect r = new Rect(inRect.x, inRect.y, Math.Min(getWidth(), inRect.width), Math.Min(getHeight(), inRect.height));
			displayToolTipIfNecessary(r, flow);
            Texture2D iconToRender = iconTexture != null ? iconTexture : exprIcon(flow);
            bool clickedBackground = false;
            if (DrawBackground)
            {
                clickedBackground = Widgets.ButtonText(r, "", true, false);
            }
            if ( iconToRender != null )
            {
                if (Widgets.ButtonImage(r, iconToRender) || clickedBackground)
                {
                    onClick?.Invoke(flow);
                }
            }
            else if (clickedBackground)
            {
                onClick?.Invoke(flow);
            }
		}
	}
}
