﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 23.10.2018
 * Time: 07:32
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using UnityEngine;
using Verse;

namespace RIMMSqol.renderers
{
	/// <summary>
	/// Description of VerticalSplitRenderer.
	/// </summary>
	public class VerticalSplitRenderer : ParentRenderer
	{
		protected Vector2 scrollPosition;
		protected BaseRenderer leftChild, rightChild;
		protected float height, widthLeft, widthRight;
		public VerticalSplitRenderer(BaseRenderer leftChild, BaseRenderer rightChild, float widthLeft = -1f, float widthRight = -1f, float height = -1f) {
			this.leftChild = leftChild;
			this.rightChild = rightChild;
			this.AddChild(leftChild).AddChild(rightChild);
			this.height = height;
			this.widthLeft = widthLeft;
			this.widthRight = widthRight;
		}
		public float getWidthLeft() {
			return ( widthLeft < 0 ? leftChild.getWidth() : widthLeft );
		}
		public float getWidthRight() {
			return ( widthRight < 0 ? rightChild.getWidth() : widthRight );
		}
		public override float getHeight() {
			if ( height < 0 )
				return Mathf.Max(leftChild.getHeight(), rightChild.getHeight());
			return height;
		}
		public override float getWidth() {
			return getWidthLeft() + getWidthRight();
		}
		public override void DoComponentContents(Rect inRect, Flow flow) {
			if ( inRect.height >= getHeight() && inRect.width >= getWidth() ) {
				DoChildren(inRect,flow);
			} else {
				//adding a scrollpane if necessary
				Rect viewRect = new Rect(0,0,getWidth(),getHeight());
				Widgets.BeginScrollView(inRect, ref scrollPosition, viewRect);
				DoChildren(viewRect,flow);
				Widgets.EndScrollView();
			}
		}
		protected void DoChildren(Rect inRect, Flow flow) {
			float wl = getWidthLeft(), h = getHeight();
			leftChild.DoComponentContents(new Rect(inRect.x,inRect.y,wl,h), flow);
			rightChild.DoComponentContents(new Rect(inRect.x+wl,inRect.y,getWidthRight(),h), flow);
		}
	}
}
