﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.10.2019
 * Time: 06:19
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using RimWorld;
using System;
using System.Collections.Generic;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerGameComponent.
	/// </summary>
	public class StorytellerGameComponent : GameComponent
	{
		protected StorytellerTree tree = new StorytellerTree();
		protected bool disableVanillaStoryteller = false;
        //TODO allowing the user to pick incidents and quests to disable in the vanilla storyteller. Checking if the quest weight selection is skipped by 
        //custom storyteller. basechance is skipped for incidents in the pick node.
        //The quests are disabled for both storyteller
        static protected Dictionary<IncidentDef, float> disabledVanillaIncidents = new Dictionary<IncidentDef, float>();
        static protected Dictionary<QuestScriptDef, float> disabledQuestScripts = new Dictionary<QuestScriptDef, float>();
        protected List<string> disabledVanillaIncidentsKeys = new List<string>(), disabledQuestScriptsKeys = new List<string>();
        public bool DisableVanillaStoryteller {
			get { return disableVanillaStoryteller; }
			set { disableVanillaStoryteller = value; }
		}
		
		public StorytellerTree Tree {
			get { return tree; }
		}

        public Dictionary<IncidentDef, float>.KeyCollection DisabledIncidents
        {
            get { return disabledVanillaIncidents.Keys; }
        }
        public Dictionary<QuestScriptDef, float>.KeyCollection DisabledQuestScripts
        {
            get { return disabledQuestScripts.Keys; }
        }

        public StorytellerGameComponent()
		{
		}
		
		public StorytellerGameComponent(Game game)
		{
			try {
				LoadStorytellerTree(DefDatabase<StorytellerTreeDef>.GetNamed("QOLNone", true));
			} catch (Exception ex) {
				Log.Error("Failed to create storyteller!\n"+ex.Message+"\n"+ex.StackTrace);
			}
		}
		
		public void LoadStorytellerTree(StorytellerTreeDef def) {
			try {
				tree.Dispose();
				tree.initializeTree(def);
			} catch (Exception ex) {
				Log.Error("Failed to create storyteller!\n"+ex.Message+"\n"+ex.StackTrace);
			}
		}
		
		public override void GameComponentTick() {
			//TODO should also register Storyteller.Notify_PawnEvent(Pawn pawn, AdaptationEvent ev, DamageInfo? dinfo = null) to allow triggering incidents via pawn events like the maninblack 
			tree.TryStartIncident();
		}
		
		public override void ExposeData() {
			Scribe_Deep.Look<StorytellerTree>(ref tree, "tree");
            if (Scribe.mode == LoadSaveMode.Saving)
            {
                if (disabledVanillaIncidentsKeys != null) disabledVanillaIncidentsKeys.Clear();
                else disabledVanillaIncidentsKeys = new List<string>();
                foreach ( IncidentDef inc in disabledVanillaIncidents.Keys )
                {
                    disabledVanillaIncidentsKeys.Add(inc.defName);
                }
                if (disabledQuestScriptsKeys != null) disabledQuestScriptsKeys.Clear();
                else disabledQuestScriptsKeys = new List<string>();
                foreach (QuestScriptDef qs in disabledQuestScripts.Keys)
                {
                    disabledQuestScriptsKeys.Add(qs.defName);
                }
            }
            Scribe_Collections.Look<string>(ref disabledVanillaIncidentsKeys, "disabledVanillaIncidents");
            Scribe_Collections.Look<string>(ref disabledQuestScriptsKeys, "disabledQuestScripts");
            if ( Scribe.mode == LoadSaveMode.ResolvingCrossRefs )
            {
                foreach (IncidentDef inc in new List<IncidentDef>(disabledVanillaIncidents.Keys))
                {
                    RemoveDisabledIncident(inc);
                }
                if (!disabledVanillaIncidentsKeys.NullOrEmpty())
                {
                    foreach ( string key in disabledVanillaIncidentsKeys )
                    {
                        IncidentDef inc = DefDatabase<IncidentDef>.GetNamedSilentFail(key);
                        if ( inc != null )
                        {
                            AddDisabledIncident(inc);
                        }
                    }
                    disabledVanillaIncidentsKeys.Clear();
                }
                foreach (QuestScriptDef qs in new List<QuestScriptDef>(disabledQuestScripts.Keys))
                {
                    RemoveDisabledQuestScript(qs);
                }
                if (!disabledQuestScriptsKeys.NullOrEmpty())
                {
                    foreach (string key in disabledQuestScriptsKeys)
                    {
                        QuestScriptDef qs = DefDatabase<QuestScriptDef>.GetNamedSilentFail(key);
                        if (qs != null)
                        {
                            AddDisabledQuestScript(qs);
                        }
                    }
                    disabledQuestScriptsKeys.Clear();
                }
            }
        }

        public void AddDisabledIncident(IncidentDef inc)
        {
            if (!disabledVanillaIncidents.ContainsKey(inc))
            {
                disabledVanillaIncidents[inc] = inc.baseChance;
                inc.baseChance = 0;
            }
        }

        public void RemoveDisabledIncident(IncidentDef inc)
        {
            float oldValue;
            if ( disabledVanillaIncidents.TryGetValue(inc, out oldValue) )
            {
                inc.baseChance = oldValue;
                disabledVanillaIncidents.Remove(inc);
            }
        }

        public void ToggleDisabledIncident(IncidentDef ic)
        {
            if (disabledVanillaIncidents.ContainsKey(ic))
                RemoveDisabledIncident(ic);
            else
                AddDisabledIncident(ic);
        }

        public void AddDisabledQuestScript(QuestScriptDef qs)
        {
            if (!disabledQuestScripts.ContainsKey(qs))
            {
                disabledQuestScripts[qs] = qs.rootSelectionWeight;
                qs.rootSelectionWeight = 0;
            }
        }

        public void RemoveDisabledQuestScript(QuestScriptDef qs)
        {
            float oldValue;
            if (disabledQuestScripts.TryGetValue(qs, out oldValue))
            {
                qs.rootSelectionWeight = oldValue;
                disabledQuestScripts.Remove(qs);
            }
        }

        public void ToggleDisabledQuestScript(QuestScriptDef qs)
        {
            if (disabledQuestScripts.ContainsKey(qs))
                RemoveDisabledQuestScript(qs);
            else
                AddDisabledQuestScript(qs);
        }

        public void MapRemoved(Map map) {
			if ( map != null && tree != null && tree.incidentQueue != null ) tree.incidentQueue.Notify_TargetLost(map);
		}
	}
}
