﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 13.11.2019
 * Time: 19:50
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_Chance.
	/// </summary>
	public class StorytellerTreeNode_Chance : StorytellerTreeNode
	{
		public SimpleCurve chanceOnDayCurve;
		public float fixedChance = -1f;
		protected List<float> curvePoints;
		
		public StorytellerTreeNode_Chance() : this(null) { }
		
		public StorytellerTreeNode_Chance(StorytellerTreeNode parent) : base(parent) { }
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			float totalChance = 1f;
			if ( chanceOnDayCurve != null && chanceOnDayCurve.PointsCount > 0 ) {
				totalChance *= chanceOnDayCurve.Evaluate(tree.lastInterval/60f);
			}
			if ( fixedChance >= 0 ) {
				totalChance *= fixedChance;
			}
			
			if ( Rand.Value < totalChance ) {
				foreach ( StorytellerTreeIncident fi in base.TryStartIncident(tree) ) yield return fi;
			}
		}
		
		public override void ExposeData() {
			base.ExposeData();
			if (Scribe.mode == LoadSaveMode.Saving) {
				curvePoints = curvePointsAsList();
			}
			Scribe_Collections.Look<float>(ref curvePoints, "chanceOnDayCurve");
			Scribe_Values.Look<float>(ref fixedChance, "fixedChance", -1f);
			if (Scribe.mode == LoadSaveMode.ResolvingCrossRefs) {
				loadCurveFromPoints(curvePoints, out chanceOnDayCurve);
				if ( curvePoints != null ) curvePoints.Clear(); 
				curvePoints = null;
			}
		}
		
		public void loadCurvePoints(List<float> curvePoints) {
			loadCurveFromPoints(curvePoints, out chanceOnDayCurve);
		}
		
		public List<float> curvePointsAsList() {
			return curvePointsAsList(chanceOnDayCurve);
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesListPrimitive("ChanceOnDayCurve","Chance on Day Curve",10,typeof(List<float>),null,
				                                                      t=>((StorytellerTreeNode_Chance)t).curvePointsAsList(),null,()=>1.0f,true));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<StorytellerTreeNode_Chance>("FixedChance","Fixed Chance",20,-1f,t=>t.fixedChance,-1f));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_Chance sttn = (StorytellerTreeNode_Chance)obj;
			sttn.loadCurvePoints(si.get<List<float>>("ChanceOnDayCurve"));
			sttn.fixedChance = si.get<float>("FixedChance");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_Chance>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_Chance : StorytellerTreeNodeConfiguration {
		public SimpleCurve chanceOnDayCurve;
		public float fixedChance;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_Chance n = new StorytellerTreeNode_Chance(parent);
			n.label = label;
			if ( chanceOnDayCurve != null && chanceOnDayCurve.PointsCount > 0 ) {
				n.chanceOnDayCurve = new SimpleCurve(); n.chanceOnDayCurve.SetPoints(chanceOnDayCurve.Points);
			} else n.chanceOnDayCurve = null;
			n.fixedChance = fixedChance;
			return n;
		}
	}
}
