﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.10.2019
 * Time: 06:25
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_Cooldown.
	/// </summary>
	public class StorytellerTreeNode_Cooldown : StorytellerTreeNode
	{
		public int spacingIntervalsMin = 0, spacingIntervalsMax = 0;
		public int waitUntilInterval = 0;
		
		protected int maxCooldownDelayThisInterval = 0;
		protected bool hadIncidentThisInterval = false;
		
		public StorytellerTreeNode_Cooldown() : this(null) { }
		
		public StorytellerTreeNode_Cooldown(StorytellerTreeNode parent) : this(parent,0) { }
		
		public StorytellerTreeNode_Cooldown(StorytellerTreeNode parent, int spacingIntervalsMin = 0, int spacingIntervalsMax = 0, bool startOnCooldown = true) : base(parent) {
			this.spacingIntervalsMin = spacingIntervalsMin;
			this.spacingIntervalsMax = spacingIntervalsMax;
			if ( startOnCooldown ) waitUntilInterval = Find.TickManager.TicksGame / 1000 + Rand.Range(spacingIntervalsMin, spacingIntervalsMax);
		}
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			if ( tree.lastInterval < waitUntilInterval ) yield break;
			
			foreach ( StorytellerTreeIncident fi in base.TryStartIncident(tree) ) {
				fi.ListenToFiredOrQueued(FiredOrQueued);
				yield return fi;
			}
			
			if ( hadIncidentThisInterval ) {
				waitUntilInterval = tree.lastInterval + (int)Math.Ceiling((double)maxCooldownDelayThisInterval / 1000) + Rand.Range(spacingIntervalsMin, spacingIntervalsMax);
				maxCooldownDelayThisInterval = 0;
				hadIncidentThisInterval = false;
			}
			
			yield break;
		}
		
		protected void FiredOrQueued(StorytellerTreeIncident fi, StorytellerTree tree) {
			hadIncidentThisInterval = true;
			maxCooldownDelayThisInterval = Math.Max(maxCooldownDelayThisInterval, fi.delay);
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<int>(ref spacingIntervalsMin, "spacingIntervalsMin");
			Scribe_Values.Look<int>(ref spacingIntervalsMax, "spacingIntervalsMax");
			Scribe_Values.Look<int>(ref waitUntilInterval, "waitUntilInterval");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_Cooldown>("SpacingIntervalsMin","Intervals-Min",10,
				                                                                                   0,n=>n.spacingIntervalsMin,0));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_Cooldown>("SpacingIntervalsMax","Intervals-Max",20,
			                                                                                   0,n=>n.spacingIntervalsMax,0));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_Cooldown>("WaitUntilInterval","Wait Until Interval",30,
			                                                                                   0,n=>n.waitUntilInterval,0));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_Cooldown sttn = (StorytellerTreeNode_Cooldown)obj;
			sttn.spacingIntervalsMin = si.get<int>("SpacingIntervalsMin");
         	sttn.spacingIntervalsMax = si.get<int>("SpacingIntervalsMax");
         	sttn.waitUntilInterval = si.get<int>("WaitUntilInterval");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_Cooldown>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_Cooldown : StorytellerTreeNodeConfiguration {
		public int spacingIntervalsMin = 0, spacingIntervalsMax = 0;
		public int waitUntilInterval = 0;
		public bool startOnCooldown = true;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_Cooldown n = new StorytellerTreeNode_Cooldown(parent, spacingIntervalsMin, spacingIntervalsMax, startOnCooldown);
			n.label = label;
			n.waitUntilInterval += waitUntilInterval;
			return n;
		}
	}
}
