﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.10.2019
 * Time: 06:24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_Incident.
	/// </summary>
	public class StorytellerTreeNode_Incident : StorytellerTreeNode
	{
		public IncidentDef incident;
		
		public override bool IsLeaf { get { return true; } }
		
		public StorytellerTreeNode_Incident() : this(null) { }
		
		public StorytellerTreeNode_Incident(StorytellerTreeNode parent) : base(parent) { }
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			StorytellerTreeComp_TargetFilter targetFilter = tree.getComp<StorytellerTreeComp_TargetFilter>();
			foreach ( IIncidentTarget target in targetFilter.FilteredTargets ) {
				if ( incident.TargetAllowed(target) ) {
					yield return new StorytellerTreeIncident(new FiringIncident(incident, null, generateParameter(incident.category, target)));
				}
			}
			yield break;
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Defs.Look<IncidentDef>(ref incident, "incident");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesSelectable("Incident","Incident",10,typeof(IncidentDef),DefDatabase<IncidentDef>.AllDefsListForReading.First(),
				                                                   t=>((StorytellerTreeNode_Incident)t).incident,
				                                                   f=>from object d in DefDatabase<IncidentDef>.AllDefsListForReading select d,
				                                                   f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<IncidentDef>("Incident",(IncidentDef)def); return true;},
				                                                   SettingsFieldPropertiesSelectable.DefLabelProducer,false));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_Incident sttn = (StorytellerTreeNode_Incident)obj;
			sttn.incident = si.get<IncidentDef>("Incident");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_Incident>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_Incident : StorytellerTreeNodeConfiguration {
		public IncidentDef incident;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_Incident n = new StorytellerTreeNode_Incident(parent);
			n.label = label;
			n.incident = incident;
			return n;
		}
	}
}
