﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 02.11.2019
 * Time: 08:16
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_IncidentCategory.
	/// </summary>
	public class StorytellerTreeNode_IncidentCategory : StorytellerTreeNode
	{
		public IncidentCategoryDef incidentCategory;
		public string pick = "random"; //"all","random","CommonalityOfDisease"
		
		public override bool IsLeaf { get { return true; } }
		
		public StorytellerTreeNode_IncidentCategory() : this(null) { }
		
		public StorytellerTreeNode_IncidentCategory(StorytellerTreeNode parent) : base(parent) { }
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			StorytellerTreeComp_TargetFilter targetFilter = tree.getComp<StorytellerTreeComp_TargetFilter>();
			List<IncidentDef> availableIncidents = new List<IncidentDef>();
			
			foreach ( IIncidentTarget target in targetFilter.FilteredTargets ) {
				availableIncidents.Clear();
				foreach ( IncidentDef incident in DefDatabase<IncidentDef>.AllDefsListForReading.Where(i=>i.category == incidentCategory) ) {
					if ( incident.TargetAllowed(target) ) {
						availableIncidents.Add(incident);
					}
				}
				
				if ( availableIncidents.Count > 0 ) {
					IncidentDef id;
					switch ( pick ) {
						case "random": id = availableIncidents.RandomElement(); availableIncidents.Clear(); availableIncidents.Add(id); break;
						case "CommonalityOfDisease": 
							if ( target.Tile != -1 ) {
								BiomeDef biome = Find.WorldGrid[target.Tile].biome;
								availableIncidents.TryRandomElementByWeight(biome.CommonalityOfDisease, out id); 
								availableIncidents.Clear(); availableIncidents.Add(id);
							} else availableIncidents.Clear();
							break;
						case "mtbDaysByBiome":
							if ( target.Tile != -1 ) {
								BiomeDef biome = Find.WorldGrid[target.Tile].biome;
								availableIncidents = new List<IncidentDef>(
									availableIncidents.Where(inc=>{
									                         	if ( inc.mtbDaysByBiome != null ) {
											                       	MTBByBiome entry = inc.mtbDaysByBiome.Find(x => x.biome == biome);
											                       	if ( entry != null ) {
											                       		return Rand.MTBEventOccurs(entry.mtbDays, 60000f, 1000f);
											                       	}
									                         	}
									                       	return false;
									                       }));
							} else availableIncidents.Clear();
							break;
					}
					
					foreach ( IncidentDef incident in availableIncidents ) {
						yield return new StorytellerTreeIncident(new FiringIncident(incident, null, generateParameter(incidentCategory, target)));
					}
				}
			}
			yield break;
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Defs.Look<IncidentCategoryDef>(ref incidentCategory, "incidentCategory");
			Scribe_Values.Look<string>(ref pick, "pick", "random");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesSelectable("IncidentCategory","Incident-Category",10,typeof(IncidentCategoryDef),DefDatabase<IncidentCategoryDef>.AllDefsListForReading.First(),
			                                                   t=>((StorytellerTreeNode_IncidentCategory)t).incidentCategory,
			                                                   f=>from object d in DefDatabase<IncidentCategoryDef>.AllDefsListForReading select d,
			                                                   f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<IncidentCategoryDef>("IncidentCategory",(IncidentCategoryDef)def); return true;},
			                                                   SettingsFieldPropertiesSelectable.DefLabelProducer,false));
			p.fields.Add(new SettingsFieldPropertiesSelectable("Pick","Pick Incident",20,typeof(string),"random",t=>((StorytellerTreeNode_IncidentCategory)t).pick,
			                                                   f=>new []{"all","random","CommonalityOfDisease","mtbDaysByBiome"},
			                                                   f=>entry=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("Pick",(string)entry); return true;},
			                                                   o=>(string)o,false));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_IncidentCategory sttn = (StorytellerTreeNode_IncidentCategory)obj;
			sttn.incidentCategory = si.get<IncidentCategoryDef>("IncidentCategory");
			sttn.pick = si.get<string>("Pick");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_IncidentCategory>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_IncidentCategory : StorytellerTreeNodeConfiguration {
		public IncidentCategoryDef incidentCategory;
		public string pick = "random";
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_IncidentCategory n = new StorytellerTreeNode_IncidentCategory(parent);
			n.label = label;
			n.incidentCategory = incidentCategory;
			n.pick = pick;
			return n;
		}
	}
}
