﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 06.12.2019
 * Time: 04:06
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeQueue.
	/// </summary>
	public class StorytellerTreeQueue : IExposable
	{
		private List<QueuedIncident> queuedIncidents = new List<QueuedIncident>();
		
		public void ExposeData()
		{
			Scribe_Collections.Look<QueuedIncident>(ref this.queuedIncidents, "queuedIncidents", LookMode.Deep, new object[0]);
		}
		
		public void Clear() {
			if ( queuedIncidents != null ) queuedIncidents.Clear();
		}
		
		public bool Add(QueuedIncident qi)
		{
			this.queuedIncidents.Add(qi);
			//ordering descending so the incidents to remove next are located at the end of the list for better performance
			//inefficent to sort the entire list when adding only one itme but lazy
			this.queuedIncidents.Sort((QueuedIncident a, QueuedIncident b) => b.FireTick.CompareTo(a.FireTick));
			/*Log.Message("CurrentQueue");
			foreach (QueuedIncident queuedIncident in queuedIncidents) {
				Log.Message("FireTick: "+queuedIncident.FireTick);
			}*/
			return true;
		}
		
		public void IncidentQueueTick()
		{
			for (int i = this.queuedIncidents.Count - 1; i >= 0; i--) {
				QueuedIncident queuedIncident = this.queuedIncidents[i];
				//once the first entry is waiting the remaining entries will too.
				if ( queuedIncident.FireTick > Find.TickManager.TicksGame ) {
					break;
				}
				if (!queuedIncident.TriedToFire) {
					if (queuedIncident.FireTick <= Find.TickManager.TicksGame) {
						queuedIncident.Notify_TriedToFire();
						FiringIncident fi = queuedIncident.FiringIncident;
						if ( (fi.parms.forced || fi.def.Worker.CanFireNow(fi.parms)) && fi.def.Worker.TryExecute(fi.parms)) {
							fi.parms.target.StoryState.Notify_IncidentFired(fi);
							this.queuedIncidents.RemoveAt(i);
							//Log.Message("Queue fired event.");
						} else if ( queuedIncident.RetryDurationTicks == 0 ) {
							this.queuedIncidents.RemoveAt(i);
							//Log.Message("Queue failed to fire event and has no retries.");
						}
					}
				} else {
					if (queuedIncident.FireTick + queuedIncident.RetryDurationTicks <= Find.TickManager.TicksGame) {
						this.queuedIncidents.RemoveAt(i);
						//Log.Message("Queue failed to fire event and has exhausted retries.");
					} else {
						if (Find.TickManager.TicksGame % 833 == Rand.RangeSeeded(0, 833, queuedIncident.FireTick)) {
							queuedIncident.Notify_TriedToFire();
							FiringIncident fi = queuedIncident.FiringIncident;
							if ( (fi.parms.forced || fi.def.Worker.CanFireNow(fi.parms)) && fi.def.Worker.TryExecute(fi.parms)) {
								fi.parms.target.StoryState.Notify_IncidentFired(fi);
								this.queuedIncidents.RemoveAt(i);
							}
						}
					}
				}
			}
		}
		
		public void Notify_TargetLost(IIncidentTarget target) {
			this.queuedIncidents.RemoveAll((QueuedIncident x) => x.FiringIncident.parms.target == target);
		}
	}
}
